@@ -8,10 +8,10 @@ from __future__ import (absolute_import, division, print_function,
 This includes tests for the Distance class and related calculations
 """
 
+import pytest
 import numpy as np
 from numpy import testing as npt
 
-from ...tests.helper import pytest
 from ... import units as u
 from .. import Longitude, Latitude, Distance, CartesianRepresentation
 from ..builtin_frames import ICRS, Galactic
@@ -35,7 +35,7 @@ def test_distances():
     coordinate.
     '''
 
-    #try all the different ways to initialize a Distance
+    # try all the different ways to initialize a Distance
     distance = Distance(12, u.parsec)
     Distance(40, unit=u.au)
     Distance(value=5, unit=u.kpc)
@@ -53,12 +53,11 @@ def test_distances():
     c = Galactic(l=158.558650*u.degree, b=-43.350066*u.degree,
                  distance=Distance(12, u.parsec))
 
-    #or initialize distances via redshifts - this is actually tested in the
-    #function below that checks for scipy. This is kept here as an example
-    #c.distance = Distance(z=0.2)  # uses current cosmology
-    #with whatever your preferred cosmology may be
-    #c.distance = Distance(z=0.2, cosmology=WMAP5)
-
+    # or initialize distances via redshifts - this is actually tested in the
+    # function below that checks for scipy. This is kept here as an example
+    # c.distance = Distance(z=0.2)  # uses current cosmology
+    # with whatever your preferred cosmology may be
+    # c.distance = Distance(z=0.2, cosmology=WMAP5)
 
     # Coordinate objects can be initialized with a distance using special
     # syntax
@@ -95,10 +94,10 @@ def test_distances():
     # TODO: choose between these when CartesianRepresentation gets a definite
     # decision on whether or not it gets __add__
     #
-    # CartesianPoints objects can be added and subtracted, which are
+    # CartesianRepresentation objects can be added and subtracted, which are
     # vector/elementwise they can also be given as arguments to a coordinate
     # system
-    #csum = ICRS(c1.cartesian + c2.cartesian)
+    # csum = ICRS(c1.cartesian + c2.cartesian)
     csumrep = CartesianRepresentation(c1.cartesian.xyz + c2.cartesian.xyz)
     csum = ICRS(csumrep)
 
@@ -118,7 +117,7 @@ def test_distances_scipy():
     """
     from ...cosmology import WMAP5
 
-    #try different ways to initialize a Distance
+    # try different ways to initialize a Distance
     d4 = Distance(z=0.23)  # uses default cosmology - as of writing, WMAP7
     npt.assert_allclose(d4.z, 0.23, rtol=1e-8)
 
@@ -128,6 +127,12 @@ def test_distances_scipy():
     d6 = Distance(z=0.23, cosmology=WMAP5, unit=u.km)
     npt.assert_allclose(d6.value, 3.5417046898762366e+22)
 
+    with pytest.raises(ValueError):
+        Distance(cosmology=WMAP5, unit=u.km)
+
+    with pytest.raises(ValueError):
+        Distance()
+
 
 def test_distance_change():
 
@@ -138,11 +143,11 @@ def test_distance_change():
     oldx = c1.cartesian.x.value
     assert (oldx - 0.35284083171901953) < 1e-10
 
-    #first make sure distances are immutible
+    # first make sure distances are immutible
     with pytest.raises(AttributeError):
         c1.distance = Distance(2, unit=u.kpc)
 
-    #now x should increase with a bigger distance increases
+    # now x should increase with a bigger distance increases
     c2 = ICRS(ra, dec, Distance(2, unit=u.kpc))
     assert c2.cartesian.x.value == oldx * 2
 
@@ -171,6 +176,11 @@ def test_distance_is_quantity():
     assert q.value[1] == 0
     assert d.value[1] != 0
 
+    # regression test against #2261
+    d = Distance([2 * u.kpc, 250. * u.pc])
+    assert d.unit is u.kpc
+    assert np.all(d.value == np.array([2., 0.25]))
+
 
 def test_distmod():
 
@@ -190,6 +200,15 @@ def test_distmod():
     with pytest.raises(ValueError):
         d = Distance(z=.23, distmod=20)
 
+    # check the Mpc/kpc/pc behavior
+    assert Distance(distmod=1).unit == u.pc
+    assert Distance(distmod=11).unit == u.kpc
+    assert Distance(distmod=26).unit == u.Mpc
+    assert Distance(distmod=-21).unit == u.AU
+
+    # if an array, uses the mean of the log of the distances
+    assert Distance(distmod=[1, 11, 26]).unit == u.kpc
+
 
 def test_distance_in_coordinates():
     """
@@ -232,7 +251,7 @@ def test_distance_comparison():
 
 
 def test_distance_to_quantity_when_not_units_of_length():
-    """Any operatation that leaves units other than those of length
+    """Any operation that leaves units other than those of length
     should turn a distance into a quantity (#2206, #2250)"""
     d = Distance(15*u.kpc)
     twice = 2.*d
