@@ -7,9 +7,10 @@ from . import core
 from ...extern import six
 from ...table import Table
 from . import cparser
-from ...extern.six.moves import zip as izip
+from ...extern.six.moves import zip
 from ...utils import set_locale
 
+
 @six.add_metaclass(core.MetaBaseReader)
 class FastBasic(object):
     """
@@ -35,7 +36,7 @@ class FastBasic(object):
             raise ValueError('header_start cannot be set to None for this Reader')
 
         kwargs = default_kwargs.copy()
-        kwargs.update(user_kwargs) # user kwargs take precedence over defaults
+        kwargs.update(user_kwargs)  # user kwargs take precedence over defaults
         delimiter = kwargs.pop('delimiter', ' ')
         self.delimiter = str(delimiter) if delimiter is not None else None
         self.write_comment = kwargs.get('comment', '# ')
@@ -80,6 +81,8 @@ class FastBasic(object):
         elif 'converters' in self.kwargs:
             raise core.ParameterError("The C reader does not support passing "
                                       "specialized converters")
+        elif 'encoding' in self.kwargs:
+            raise core.ParameterError("The C reader does not use the encoding parameter")
         elif 'Outputter' in self.kwargs:
             raise core.ParameterError("The C reader does not use the Outputter parameter")
         elif 'Inputter' in self.kwargs:
@@ -152,6 +155,7 @@ class FastBasic(object):
         writer = cparser.FastWriter(table, **write_kwargs)
         writer.write(output, header_output, output_types)
 
+
 class FastCsv(FastBasic):
     """
     A faster version of the ordinary :class:`Csv` writer that uses the
@@ -172,7 +176,8 @@ class FastCsv(FastBasic):
         Override the default write method of `FastBasic` to
         output masked values as empty fields.
         """
-        self._write(table, output, { 'fill_values': [(core.masked, '')] })
+        self._write(table, output, {'fill_values': [(core.masked, '')]})
+
 
 class FastTab(FastBasic):
     """
@@ -188,11 +193,12 @@ class FastTab(FastBasic):
         self.strip_whitespace_lines = False
         self.strip_whitespace_fields = False
 
+
 class FastNoHeader(FastBasic):
     """
     This class uses the fast C engine to read tables with no header line. If
     the names parameter is unspecified, the columns will be autonamed with
-    "col%d".
+    "col{}".
     """
     _format_name = 'fast_no_header'
     _description = 'Basic table with no headers using the fast C engine'
@@ -208,6 +214,7 @@ class FastNoHeader(FastBasic):
         """
         self._write(table, output, {}, header_output=None)
 
+
 class FastCommentedHeader(FastBasic):
     """
     A faster version of the :class:`CommentedHeader` reader, which looks for
@@ -232,10 +239,13 @@ class FastCommentedHeader(FastBasic):
         """
         out = super(FastCommentedHeader, self).read(table)
 
-        # Strip off first comment since this is the header line for
-        # commented_header format.
+        # Strip off the comment line set as the header line for
+        # commented_header format (first by default).
         if 'comments' in out.meta:
-            out.meta['comments'] = out.meta['comments'][1:]
+            idx = self.header_start
+            if idx < 0:
+                idx = len(out.meta['comments']) + idx
+            out.meta['comments'] = out.meta['comments'][:idx] + out.meta['comments'][idx+1:]
             if not out.meta['comments']:
                 del out.meta['comments']
 
@@ -247,7 +257,7 @@ class FastCommentedHeader(FastBasic):
 
         for line in tmp.splitlines():
             line = line.lstrip()
-            if line and line[0] == self.comment: # line begins with a comment
+            if line and line[0] == self.comment:  # line begins with a comment
                 commented_lines.append(line[1:])
                 if len(commented_lines) == self.header_start + 1:
                     break
@@ -267,6 +277,7 @@ class FastCommentedHeader(FastBasic):
         """
         self._write(table, output, {}, header_output='comment')
 
+
 class FastRdb(FastBasic):
     """
     A faster version of the :class:`Rdb` reader. This format is similar to
@@ -293,7 +304,7 @@ class FastRdb(FastBasic):
             elif not line2 and line.strip() and line.lstrip()[0] != self.comment:
                 line2 = line
                 break
-        else: # less than 2 lines in table
+        else:  # less than 2 lines in table
             raise ValueError('RDB header requires 2 lines')
 
         # tokenize the two header lines separately
@@ -317,7 +328,7 @@ class FastRdb(FastBasic):
         try_float = {}
         try_string = {}
 
-        for name, col_type in izip(self.engine.get_names(), types):
+        for name, col_type in zip(self.engine.get_names(), types):
             if col_type[-1].lower() == 's':
                 try_int[name] = 0
                 try_float[name] = 0
