@@ -1,17 +1,19 @@
+# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
 # TEST_UNICODE_LITERALS
 
 import re
-from io import BytesIO
+from io import BytesIO, open
 from collections import OrderedDict
 import locale
-
-import numpy as np
 import platform
 
-from ....extern.six.moves import cStringIO as StringIO
-from ....tests.helper import pytest
+import pytest
+import numpy as np
+
+from ....extern import six  # noqa
+from ....extern.six.moves import zip, cStringIO as StringIO
 from ... import ascii
 from ....table import Table
 from ....units import Unit
@@ -36,11 +38,14 @@ else:
     HAS_PATHLIB = True
 
 
-@pytest.mark.parametrize('fast_reader', [True, False, 'force'])
+@pytest.mark.parametrize('fast_reader', [True, False, {'use_fast_converter': False},
+                                         {'use_fast_converter': True}, 'force'])
 def test_convert_overflow(fast_reader):
     """
     Test reading an extremely large integer, which falls through to
-    string due to an overflow error (#2234).
+    string due to an overflow error (#2234). The C parsers used to
+    return inf (kind 'f') for this.
+    Kind should be 'S' in Python2, 'U' in Python3.
     """
     expected_kind = ('S', 'U')
     dat = ascii.read(['a', '1' * 10000], format='basic',
@@ -117,16 +122,16 @@ def test_read_with_names_arg(fast_reader):
 def test_read_all_files(fast_reader):
     for testfile in get_testfiles():
         if testfile.get('skip'):
-            print('\n\n******** SKIPPING %s' % testfile['name'])
+            print('\n\n******** SKIPPING {}'.format(testfile['name']))
             continue
-        print('\n\n******** READING %s' % testfile['name'])
+        print('\n\n******** READING {}'.format(testfile['name']))
         for guess in (True, False):
             test_opts = testfile['opts'].copy()
             if 'guess' not in test_opts:
                 test_opts['guess'] = guess
             if 'Reader' in test_opts and 'fast_{0}'.format(test_opts['Reader']._format_name) \
-                in core.FAST_CLASSES: # has fast version
-                if 'Inputter' not in test_opts: # fast reader doesn't allow this
+                in core.FAST_CLASSES:  # has fast version
+                if 'Inputter' not in test_opts:  # fast reader doesn't allow this
                     test_opts['fast_reader'] = fast_reader
             table = ascii.read(testfile['name'], **test_opts)
             assert_equal(table.dtype.names, testfile['cols'])
@@ -138,9 +143,9 @@ def test_read_all_files(fast_reader):
 def test_read_all_files_via_table(fast_reader):
     for testfile in get_testfiles():
         if testfile.get('skip'):
-            print('\n\n******** SKIPPING %s' % testfile['name'])
+            print('\n\n******** SKIPPING {}'.format(testfile['name']))
             continue
-        print('\n\n******** READING %s' % testfile['name'])
+        print('\n\n******** READING {}'.format(testfile['name']))
         for guess in (True, False):
             test_opts = testfile['opts'].copy()
             if 'guess' not in test_opts:
@@ -161,11 +166,11 @@ def test_read_all_files_via_table(fast_reader):
 def test_guess_all_files():
     for testfile in get_testfiles():
         if testfile.get('skip'):
-            print('\n\n******** SKIPPING %s' % testfile['name'])
+            print('\n\n******** SKIPPING {}'.format(testfile['name']))
             continue
         if not testfile['opts'].get('guess', True):
             continue
-        print('\n\n******** READING %s' % testfile['name'])
+        print('\n\n******** READING {}'.format(testfile['name']))
         for filter_read_opts in (['Reader', 'delimiter', 'quotechar'], []):
             # Copy read options except for those in filter_read_opts
             guess_opts = dict((k, v) for k, v in testfile['opts'].items()
@@ -202,7 +207,7 @@ def test_daophot_header_keywords():
     table = ascii.read('t/daophot.dat', Reader=ascii.Daophot)
     expected_keywords = (('NSTARFILE', 'test.nst.1', 'filename', '%-23s'),
                          ('REJFILE', '"hello world"', 'filename', '%-23s'),
-                         ('SCALE', '1.',  'units/pix', '%-23.7g'),)
+                         ('SCALE', '1.', 'units/pix', '%-23.7g'),)
 
     keywords = table.meta['keywords']  # Ordered dict of keyword structures
     for name, value, units, format_ in expected_keywords:
@@ -349,7 +354,7 @@ def test_set_converters():
                   }
     data = ascii.read('t/test4.dat', converters=converters)
     assert_equal(str(data['zabs1.nh'].dtype), 'float32')
-    assert_equal(data['p1.gamma'][0], '1.26764544642')
+    assert_equal(data['p1.gamma'][0], '1.26764500000')
 
 
 @pytest.mark.parametrize('fast_reader', [True, False, 'force'])
@@ -417,7 +422,7 @@ def test_fill_values_include_names(fast_reader):
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
     data = ascii.read(f, fill_values=('a', '1'), fast_reader=fast_reader,
-                           fill_include_names = ['b'], **testfile['opts'])
+                           fill_include_names=['b'], **testfile['opts'])
     check_fill_values(data)
 
 
@@ -426,7 +431,7 @@ def test_fill_values_exclude_names(fast_reader):
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
     data = ascii.read(f, fill_values=('a', '1'), fast_reader=fast_reader,
-                           fill_exclude_names = ['a'], **testfile['opts'])
+                           fill_exclude_names=['a'], **testfile['opts'])
     check_fill_values(data)
 
 
@@ -770,13 +775,13 @@ def get_testfiles(name=None):
         {'name': 't/whitespace.dat',
          'cols': ('quoted colname with tab\tinside', 'col2', 'col3'),
          'nrows': 2,
-         'opts': {'delimiter': '\s'}},
+         'opts': {'delimiter': r'\s'}},
         {'name': 't/simple_csv.csv',
-         'cols': ('a','b','c'),
+         'cols': ('a', 'b', 'c'),
          'nrows': 2,
          'opts': {'Reader': ascii.Csv}},
         {'name': 't/simple_csv_missing.csv',
-         'cols': ('a','b','c'),
+         'cols': ('a', 'b', 'c'),
          'nrows': 2,
          'skip': True,
          'opts': {'Reader': ascii.Csv}},
@@ -812,6 +817,7 @@ def get_testfiles(name=None):
     else:
         return testfiles
 
+
 def test_header_start_exception():
     '''Check certain Readers throw an exception if ``header_start`` is set
 
@@ -836,6 +842,7 @@ def test_csv_table_read():
     t = ascii.read(lines)
     assert t.colnames == ['a', 'b']
 
+
 @pytest.mark.parametrize('fast_reader', [True, False, 'force'])
 def test_overlapping_names(fast_reader):
     """
@@ -845,6 +852,7 @@ def test_overlapping_names(fast_reader):
     t = ascii.read(['a b', '1 2'], names=['b', 'a'], fast_reader=fast_reader)
     assert t.colnames == ['b', 'a']
 
+
 def test_sextractor_units():
     """
     Make sure that the SExtractor reader correctly inputs descriptions and units.
@@ -866,6 +874,7 @@ def test_sextractor_units():
         assert table[colname].unit == expected_units[i]
         assert table[colname].description == expected_descrs[i]
 
+
 def test_sextractor_last_column_array():
     """
     Make sure that the SExtractor reader handles the last column correctly when it is array-like.
@@ -883,7 +892,7 @@ def test_sextractor_last_column_array():
                        'Right ascension of barycenter (J2000)',
                        'Declination of barycenter (J2000)',
                        'Kron-like elliptical aperture magnitude',
-                       'RMS error for AUTO magnitude',] + [
+                       'RMS error for AUTO magnitude', ] + [
                        'Fixed aperture magnitude vector'] * 7 + [
                        'RMS error vector for fixed aperture mag.'] * 7
     for i, colname in enumerate(table.colnames):
@@ -891,6 +900,7 @@ def test_sextractor_last_column_array():
         assert table[colname].unit == expected_units[i]
         assert table[colname].description == expected_descrs[i]
 
+
 def test_list_with_newlines():
     """
     Check that lists of strings where some strings consist of just a newline
@@ -902,6 +912,7 @@ def test_list_with_newlines():
     assert t[0][0] == 123
     assert t[1][0] == 456
 
+
 def test_commented_csv():
     """
     Check that Csv reader does not have ignore lines with the # comment
@@ -912,6 +923,7 @@ def test_commented_csv():
     assert len(t) == 2
     assert t['#a'][1] == '#3'
 
+
 def test_meta_comments():
     """
     Make sure that line comments are included in the ``meta`` attribute
@@ -921,6 +933,7 @@ def test_meta_comments():
     assert t.colnames == ['a', 'b', 'c']
     assert t.meta['comments'] == ['comment1', 'comment2']
 
+
 def test_guess_fail():
     """
     Check the error message when guess fails
@@ -946,7 +959,7 @@ def test_guessing_file_object():
     Test guessing a file object.  Fixes #3013 and similar issue noted in #3019.
     """
     t = ascii.read(open('t/ipac.dat.bz2', 'rb'))
-    assert t.colnames == ['ra','dec','sai','v2','sptype']
+    assert t.colnames == ['ra', 'dec', 'sai', 'v2', 'sptype']
 
 
 def test_pformat_roundtrip():
@@ -993,29 +1006,51 @@ def test_almost_but_not_quite_daophot():
     assert len(dat) == 3
 
 
-@pytest.mark.parametrize('fast', [True, False])
+@pytest.mark.parametrize('fast', [False, 'force'])
 def test_commented_header_comments(fast):
     """
-    Test that comments in commented_header are as expected and that the
-    table round-trips.
+    Test that comments in commented_header are as expected with header_start
+    at different positions, and that the table round-trips.
     """
+    comments = ['comment 1', 'comment 2', 'comment 3']
     lines = ['# a b',
              '# comment 1',
              '# comment 2',
+             '# comment 3',
              '1 2',
              '3 4']
     dat = ascii.read(lines, format='commented_header', fast_reader=fast)
-    assert dat.meta['comments'] == ['comment 1', 'comment 2']
+    assert dat.meta['comments'] == comments
+    assert dat.colnames == ['a', 'b']
 
     out = StringIO()
     ascii.write(dat, out, format='commented_header', fast_writer=fast)
     assert out.getvalue().splitlines() == lines
 
+    lines.insert(1, lines.pop(0))
+    dat = ascii.read(lines, format='commented_header', header_start=1, fast_reader=fast)
+    assert dat.meta['comments'] == comments
+    assert dat.colnames == ['a', 'b']
+
+    lines.insert(2, lines.pop(1))
+    dat = ascii.read(lines, format='commented_header', header_start=2, fast_reader=fast)
+    assert dat.meta['comments'] == comments
+    assert dat.colnames == ['a', 'b']
+    dat = ascii.read(lines, format='commented_header', header_start=-2, fast_reader=fast)
+    assert dat.meta['comments'] == comments
+    assert dat.colnames == ['a', 'b']
+
+    lines.insert(3, lines.pop(2))
+    dat = ascii.read(lines, format='commented_header', header_start=-1, fast_reader=fast)
+    assert dat.meta['comments'] == comments
+    assert dat.colnames == ['a', 'b']
+
     lines = ['# a b',
              '1 2',
              '3 4']
     dat = ascii.read(lines, format='commented_header', fast_reader=fast)
     assert 'comments' not in dat.meta
+    assert dat.colnames == ['a', 'b']
 
 
 def test_probably_html():
@@ -1030,7 +1065,7 @@ def test_probably_html():
                   'file://blah.com/table.htm',
                   ' <! doctype html > hello world',
                   'junk < table baz> <tr foo > <td bar> </td> </tr> </table> junk',
-                  ['junk < table baz>', ' <tr foo >', ' <td bar> ', '</td> </tr>',  '</table> junk'],
+                  ['junk < table baz>', ' <tr foo >', ' <td bar> ', '</td> </tr>', '</table> junk'],
                   (' <! doctype html > ', ' hello world'),
     ):
         assert _probably_html(table) is True
@@ -1041,7 +1076,7 @@ def test_probably_html():
                   'fole://blah/table.htm',
                   ' < doctype html > hello world',
                   'junk < tble baz> <tr foo > <td bar> </td> </tr> </table> junk',
-                  ['junk < table baz>', ' <t foo >', ' <td bar> ', '</td> </tr>',  '</table> junk'],
+                  ['junk < table baz>', ' <t foo >', ' <td bar> ', '</td> </tr>', '</table> junk'],
                   (' <! doctype htm > ', ' hello world'),
                   [[1, 2, 3]],
     ):
@@ -1127,6 +1162,7 @@ def test_table_with_no_newline():
         assert t.colnames == ['a', 'b']
         assert len(t) == 0
 
+
 @pytest.mark.skipif('not HAS_PATHLIB')
 def test_path_object():
     fpath = pathlib.Path('t/simple.txt')
@@ -1155,6 +1191,7 @@ def test_column_conversion_error():
         ascii.read(['a b', '1 2'], guess=False, format='basic', converters={'a': []})
     assert 'no converters' in str(err.value)
 
+
 def test_non_C_locale_with_fast_reader():
     """Test code that forces "C" locale while calling fast reader (#4364)"""
     current = locale.setlocale(locale.LC_ALL)
@@ -1182,4 +1219,103 @@ def test_no_units_for_char_columns():
     out = StringIO()
     ascii.write(t1, out, format="ipac")
     t2 = ascii.read(out.getvalue(), format="ipac", guess=False)
-    assert t2["B"].unit == None
+    assert t2["B"].unit is None
+
+
+def test_initial_column_fill_values():
+    """Regression test for #5336, #5338."""
+
+    class TestHeader(ascii.BasicHeader):
+        def _set_cols_from_names(self):
+            self.cols = [ascii.Column(name=x) for x in self.names]
+            # Set some initial fill values
+            for col in self.cols:
+                col.fill_values = {'--': '0'}
+
+    class Tester(ascii.Basic):
+        header_class = TestHeader
+
+    reader = ascii.get_reader(Reader=Tester)
+
+    assert reader.read("""# Column definition is the first uncommented line
+# Default delimiter is the space character.
+a b c
+# Data starts after the header column definition, blank lines ignored
+-- 2 3
+4 5 6 """)['a'][0] is np.ma.masked
+
+
+def test_latex_no_trailing_backslash():
+    """
+    Test that latex/aastex file with no trailing backslash can be read.
+    """
+    lines = r"""
+\begin{table}
+\begin{tabular}{ccc}
+a & b & c \\
+1 & 1.0 & c \\ % comment
+3\% & 3.0 & e  % comment
+\end{tabular}
+\end{table}
+"""
+    dat = ascii.read(lines, format='latex')
+    assert dat.colnames == ['a', 'b', 'c']
+    assert np.all(dat['a'] == ['1', r'3\%'])
+    assert np.all(dat['c'] == ['c', 'e'])
+
+
+def text_aastex_no_trailing_backslash():
+    lines = r"""
+\begin{deluxetable}{ccc}
+\tablehead{\colhead{a} & \colhead{b} & \colhead{c}}
+\startdata
+1 & 1.0 & c \\
+2 & 2.0 & d \\ % comment
+3\% & 3.0 & e  % comment
+\enddata
+\end{deluxetable}
+"""
+    dat = ascii.read(lines, format='aastex')
+    assert dat.colnames == ['a', 'b', 'c']
+    assert np.all(dat['a'] == ['1', r'3\%'])
+    assert np.all(dat['c'] == ['c', 'e'])
+
+
+@pytest.mark.skipif('six.PY2')
+@pytest.mark.parametrize('encoding', ['utf8', 'latin1', 'cp1252'])
+def test_read_with_encoding(tmpdir, encoding):
+    data = {
+        'commented_header': u'# à b è \n 1 2 héllo',
+        'csv': u'à,b,è\n1,2,héllo'
+    }
+
+    testfile = str(tmpdir.join('test.txt'))
+    for fmt, content in data.items():
+        with open(testfile, 'w', encoding=encoding) as f:
+            f.write(content)
+
+        table = ascii.read(testfile, encoding=encoding)
+        assert table.pformat() == [' à   b    è  ',
+                                   '--- --- -----',
+                                   '  1   2 héllo']
+
+        for guess in (True, False):
+            table = ascii.read(testfile, format=fmt, fast_reader=False,
+                               encoding=encoding, guess=guess)
+            assert table['è'].dtype.kind == 'U'
+            assert table.pformat() == [' à   b    è  ',
+                                       '--- --- -----',
+                                       '  1   2 héllo']
+
+
+def test_unsupported_read_with_encoding(tmpdir):
+    # Fast reader is not supported, make sure it raises an exception
+    with pytest.raises(ascii.ParameterError):
+        ascii.read('t/simple3.txt', guess=False, fast_reader='force',
+                   encoding='latin1', format='fast_csv')
+
+    # Python 2 is not supported, make sure it raises an exception
+    if six.PY2:
+        with pytest.raises(ValueError):
+            ascii.read('t/simple3.txt', guess=False, fast_reader=False,
+                       encoding='latin1', format='csv')
