@@ -2,6 +2,8 @@
 
 from __future__ import division, with_statement
 
+import os
+import shutil
 import warnings
 
 import pytest
@@ -45,6 +47,8 @@ class TestConvenience(FitsTestCase):
         header = fits.getheader(f)
         assert not f.closed
 
+        f.close()  # Close it now
+
     def test_table_to_hdu(self):
         table = Table([[1, 2, 3], ['a', 'b', 'c'], [2.3, 4.5, 6.7]],
                       names=['a', 'b', 'c'], dtype=['i', 'U1', 'f'])
@@ -141,3 +145,27 @@ class TestConvenience(FitsTestCase):
 
                 with pytest.raises(NotImplementedError):
                     printdiff(in1, in2, 0)
+
+    def test_tabledump(self):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/6937
+        """
+        # copy fits file to the temp directory
+        filename = self.data('tb.fits')
+        temp_filename = self.temp('tb.fits')
+        shutil.copyfile(filename, temp_filename)
+
+        # test without datafile
+        fits.tabledump(temp_filename)
+        assert os.path.isfile(self.temp('tb_1.txt'))
+
+        # test with datafile
+        fits.tabledump(temp_filename, datafile=self.temp('test_tb.txt'))
+        assert os.path.isfile(self.temp('test_tb.txt'))
+
+    @pytest.mark.parametrize('mode', ['wb', 'wb+', 'ab', 'ab+'])
+    def test_append_filehandle(self, tmpdir, mode):
+
+        append_file = tmpdir.join('append.fits')
+        with append_file.open(mode) as handle:
+            fits.append(filename=handle, data=np.ones((4, 4)))
