@@ -2,13 +2,14 @@
 from __future__ import division, with_statement
 
 import gzip
+import bz2
 import io
 import mmap
 import os
-import shutil
 import warnings
 import zipfile
 
+import pytest
 import numpy as np
 
 try:
@@ -17,33 +18,41 @@ try:
 except ImportError:
     HAVE_STRINGIO = False
 
-from ....io import fits
-from ....tests.helper import pytest, raises, catch_warnings
-from ....utils.exceptions import AstropyDeprecationWarning
 from . import FitsTestCase
-from .util import ignore_warnings
+
 from ..convenience import _getext
 from ..diff import FITSDiff
 from ..file import _File, GZIP_MAGIC
 
+from ....extern.six.moves import range, zip
+from ....io import fits
+from ....tests.helper import raises, catch_warnings, ignore_warnings
+from ....utils.data import get_pkg_data_filename
+from ....utils.exceptions import AstropyUserWarning
+from ....utils import data
+
+
+try:
+    import pathlib
+except ImportError:
+    HAS_PATHLIB = False
+else:
+    HAS_PATHLIB = True
+
 
 class TestCore(FitsTestCase):
-    def test_with_statement(self):
-        with fits.open(self.data('ascii.fits')) as f:
-            pass
 
     @raises(IOError)
     def test_missing_file(self):
         fits.open(self.temp('does-not-exist.fits'))
 
     def test_naxisj_check(self):
-        hdulist = fits.open(self.data('o4sp040b0_raw.fits'))
+        with fits.open(self.data('o4sp040b0_raw.fits')) as hdulist:
+            hdulist[1].header['NAXIS3'] = 500
 
-        hdulist[1].header['NAXIS3'] = 500
-
-        assert 'NAXIS3' in hdulist[1].header
-        hdulist.verify('silentfix')
-        assert 'NAXIS3' not in hdulist[1].header
+            assert 'NAXIS3' in hdulist[1].header
+            hdulist.verify('silentfix')
+            assert 'NAXIS3' not in hdulist[1].header
 
     def test_byteswap(self):
         p = fits.PrimaryHDU()
@@ -61,11 +70,24 @@ class TestCore(FitsTestCase):
         l.append(p)
         l.append(t)
 
-        l.writeto(self.temp('test.fits'), clobber=True)
+        l.writeto(self.temp('test.fits'), overwrite=True)
 
         with fits.open(self.temp('test.fits')) as p:
             assert p[1].data[1]['foo'] == 60000.0
 
+    @pytest.mark.skipif('not HAS_PATHLIB')
+    def test_fits_file_path_object(self):
+        """
+        Testing when fits file is passed as pathlib.Path object #4412.
+        """
+        fpath = pathlib.Path(get_pkg_data_filename('data/tdim.fits'))
+        with fits.open(fpath) as hdulist:
+            assert hdulist[0].filebytes() == 2880
+            assert hdulist[1].filebytes() == 5760
+
+            with fits.open(self.data('tdim.fits')) as hdulist2:
+                assert FITSDiff(hdulist2, hdulist).identical is True
+
     def test_add_del_columns(self):
         p = fits.ColDefs([])
         p.add_col(fits.Column(name='FOO', format='3J'))
@@ -91,7 +113,7 @@ class TestCore(FitsTestCase):
         assert table.data.dtype.names == ('c2', 'c4', 'foo')
         assert table.columns.names == ['c2', 'c4', 'foo']
 
-        hdulist.writeto(self.temp('test.fits'), clobber=True)
+        hdulist.writeto(self.temp('test.fits'), overwrite=True)
         with ignore_warnings():
             # TODO: The warning raised by this test is actually indication of a
             # bug and should *not* be ignored. But as it is a known issue we
@@ -102,7 +124,6 @@ class TestCore(FitsTestCase):
                 assert table.data.dtype.names == ('c2', 'c4', 'foo')
                 assert table.columns.names == ['c2', 'c4', 'foo']
 
-    @ignore_warnings(AstropyDeprecationWarning)
     def test_update_header_card(self):
         """A very basic test for the Header.update method--I'd like to add a
         few more cases to this at some point.
@@ -113,23 +134,12 @@ class TestCore(FitsTestCase):
         header['BITPIX'] = (16, comment)
         assert 'BITPIX' in header
         assert header['BITPIX'] == 16
-        assert header.ascard['BITPIX'].comment == comment
+        assert header.comments['BITPIX'] == comment
 
-        # The new API doesn't support savecomment so leave this line here; at
-        # any rate good to have testing of the new API mixed with the old API
-        header.update('BITPIX', 32, savecomment=True)
-        # Make sure the value has been updated, but the comment was preserved
+        header.update(BITPIX=32)
         assert header['BITPIX'] == 32
-        assert header.ascard['BITPIX'].comment == comment
+        assert header.comments['BITPIX'] == ''
 
-        # The comment should still be preserved--savecomment only takes effect
-        # if a new comment is also specified
-        header['BITPIX'] = 16
-        assert header.ascard['BITPIX'].comment == comment
-        header.update('BITPIX', 16, 'foobarbaz', savecomment=True)
-        assert header.ascard['BITPIX'].comment == comment
-
-    @ignore_warnings(AstropyDeprecationWarning)
     def test_set_card_value(self):
         """Similar to test_update_header_card(), but tests the the
         `header['FOO'] = 'bar'` method of updating card values.
@@ -137,33 +147,32 @@ class TestCore(FitsTestCase):
 
         header = fits.Header()
         comment = 'number of bits per data pixel'
-        card = fits.Card.fromstring('BITPIX  = 32 / %s' % comment)
-        header.ascard.append(card)
+        card = fits.Card.fromstring('BITPIX  = 32 / {}'.format(comment))
+        header.append(card)
 
         header['BITPIX'] = 32
 
         assert 'BITPIX' in header
         assert header['BITPIX'] == 32
-        assert header.ascard['BITPIX'].key == 'BITPIX'
-        assert header.ascard['BITPIX'].value == 32
-        assert header.ascard['BITPIX'].comment == comment
+        assert header.cards[0].keyword == 'BITPIX'
+        assert header.cards[0].value == 32
+        assert header.cards[0].comment == comment
 
     def test_uint(self):
-        hdulist_f = fits.open(self.data('o4sp040b0_raw.fits'))
-        hdulist_i = fits.open(self.data('o4sp040b0_raw.fits'), uint=True)
+        filename = self.data('o4sp040b0_raw.fits')
+        with fits.open(filename, uint=False) as hdulist_f:
+            with fits.open(filename, uint=True) as hdulist_i:
+                assert hdulist_f[1].data.dtype == np.float32
+                assert hdulist_i[1].data.dtype == np.uint16
+                assert np.all(hdulist_f[1].data == hdulist_i[1].data)
 
-        assert hdulist_f[1].data.dtype == np.float32
-        assert hdulist_i[1].data.dtype == np.uint16
-        assert np.all(hdulist_f[1].data == hdulist_i[1].data)
-
-    @ignore_warnings(AstropyDeprecationWarning)
     def test_fix_missing_card_append(self):
         hdu = fits.ImageHDU()
         errs = hdu.req_cards('TESTKW', None, None, 'foo', 'silentfix', [])
         assert len(errs) == 1
         assert 'TESTKW' in hdu.header
         assert hdu.header['TESTKW'] == 'foo'
-        assert hdu.header.ascard[-1].key == 'TESTKW'
+        assert hdu.header.cards[-1].keyword == 'TESTKW'
 
     def test_fix_invalid_keyword_value(self):
         hdu = fits.ImageHDU()
@@ -217,9 +226,9 @@ class TestCore(FitsTestCase):
             del hdu.header['NAXIS']
             try:
                 hdu.verify('ignore')
-            except Exception as e:
+            except Exception as exc:
                 self.fail('An exception occurred when the verification error '
-                          'should have been ignored: %s' % exc)
+                          'should have been ignored: {}'.format(exc))
         # Make sure the error wasn't fixed either, silently or otherwise
         assert 'NAXIS' not in hdu.header
 
@@ -251,7 +260,7 @@ class TestCore(FitsTestCase):
                 hdu.verify('silentfix+ignore')
             except Exception as exc:
                 self.fail('An exception occurred when the verification error '
-                          'should have been ignored: %s' % exc)
+                          'should have been ignored: {}'.format(exc))
 
         # silentfix+warn should be quiet about the fixed HDU and only warn
         # about the unfixable one
@@ -303,57 +312,56 @@ class TestCore(FitsTestCase):
         Test the various different ways of specifying an extension header in
         the convenience functions.
         """
-
-        hl, ext = _getext(self.data('test0.fits'), 'readonly', 1)
+        filename = self.data('test0.fits')
+        hl, ext = _getext(filename, 'readonly', 1)
         assert ext == 1
-        pytest.raises(ValueError, _getext, self.data('test0.fits'), 'readonly',
+        pytest.raises(ValueError, _getext, filename, 'readonly',
                       1, 2)
-        pytest.raises(ValueError, _getext, self.data('test0.fits'), 'readonly',
+        pytest.raises(ValueError, _getext, filename, 'readonly',
                       (1, 2))
-        pytest.raises(ValueError, _getext, self.data('test0.fits'), 'readonly',
+        pytest.raises(ValueError, _getext, filename, 'readonly',
                       'sci', 'sci')
-        pytest.raises(TypeError, _getext, self.data('test0.fits'), 'readonly',
+        pytest.raises(TypeError, _getext, filename, 'readonly',
                       1, 2, 3)
-        hl, ext = _getext(self.data('test0.fits'), 'readonly', ext=1)
+        hl, ext = _getext(filename, 'readonly', ext=1)
         assert ext == 1
-        hl, ext = _getext(self.data('test0.fits'), 'readonly', ext=('sci', 2))
+        hl, ext = _getext(filename, 'readonly', ext=('sci', 2))
         assert ext == ('sci', 2)
-        pytest.raises(TypeError, _getext, self.data('test0.fits'), 'readonly',
+        pytest.raises(TypeError, _getext, filename, 'readonly',
                       1, ext=('sci', 2), extver=3)
-        pytest.raises(TypeError, _getext, self.data('test0.fits'), 'readonly',
+        pytest.raises(TypeError, _getext, filename, 'readonly',
                       ext=('sci', 2), extver=3)
 
-        hl, ext = _getext(self.data('test0.fits'), 'readonly', 'sci')
+        hl, ext = _getext(filename, 'readonly', 'sci')
         assert ext == ('sci', 1)
-        hl, ext = _getext(self.data('test0.fits'), 'readonly', 'sci', 1)
+        hl, ext = _getext(filename, 'readonly', 'sci', 1)
         assert ext == ('sci', 1)
-        hl, ext = _getext(self.data('test0.fits'), 'readonly', ('sci', 1))
+        hl, ext = _getext(filename, 'readonly', ('sci', 1))
         assert ext == ('sci', 1)
-        hl, ext = _getext(self.data('test0.fits'), 'readonly', 'sci',
+        hl, ext = _getext(filename, 'readonly', 'sci',
                           extver=1, do_not_scale_image_data=True)
         assert ext == ('sci', 1)
-        pytest.raises(TypeError, _getext, self.data('test0.fits'), 'readonly',
+        pytest.raises(TypeError, _getext, filename, 'readonly',
                       'sci', ext=1)
-        pytest.raises(TypeError, _getext, self.data('test0.fits'), 'readonly',
+        pytest.raises(TypeError, _getext, filename, 'readonly',
                       'sci', 1, extver=2)
 
-        hl, ext = _getext(self.data('test0.fits'), 'readonly', extname='sci')
+        hl, ext = _getext(filename, 'readonly', extname='sci')
         assert ext == ('sci', 1)
-        hl, ext = _getext(self.data('test0.fits'), 'readonly', extname='sci',
+        hl, ext = _getext(filename, 'readonly', extname='sci',
                           extver=1)
         assert ext == ('sci', 1)
-        pytest.raises(TypeError, _getext, self.data('test0.fits'), 'readonly',
+        pytest.raises(TypeError, _getext, filename, 'readonly',
                       extver=1)
 
+        hl.close()  # Close file handler
+
     def test_extension_name_case_sensitive(self):
         """
         Tests that setting fits.conf.extension_name_case_sensitive at
         runtime works.
         """
 
-        if 'PYFITS_EXTENSION_NAME_CASE_SENSITIVE' in os.environ:
-            del os.environ['PYFITS_EXTENSION_NAME_CASE_SENSITIVE']
-
         hdu = fits.ImageHDU()
         hdu.name = 'sCi'
         assert hdu.name == 'SCI'
@@ -374,11 +382,14 @@ class TestCore(FitsTestCase):
         Tests creating a fully-formed HDU object from a string containing the
         bytes of the HDU.
         """
+        infile = self.data('test0.fits')
+        outfile = self.temp('test.fits')
 
-        dat = open(self.data('test0.fits'), 'rb').read()
+        with open(infile, 'rb') as fin:
+            dat = fin.read()
 
         offset = 0
-        with fits.open(self.data('test0.fits')) as hdul:
+        with fits.open(infile) as hdul:
             hdulen = hdul[0]._data_offset + hdul[0]._data_size
             hdu = fits.PrimaryHDU.fromstring(dat[:hdulen])
             assert isinstance(hdu, fits.PrimaryHDU)
@@ -386,14 +397,14 @@ class TestCore(FitsTestCase):
             assert hdu.data is None
 
         hdu.header['TEST'] = 'TEST'
-        hdu.writeto(self.temp('test.fits'))
-        with fits.open(self.temp('test.fits')) as hdul:
+        hdu.writeto(outfile)
+        with fits.open(outfile) as hdul:
             assert isinstance(hdu, fits.PrimaryHDU)
             assert hdul[0].header[:-1] == hdu.header[:-1]
             assert hdul[0].header['TEST'] == 'TEST'
             assert hdu.data is None
 
-        with fits.open(self.data('test0.fits'))as hdul:
+        with fits.open(infile)as hdul:
             for ext_hdu in hdul[1:]:
                 offset += hdulen
                 hdulen = len(str(ext_hdu.header)) + ext_hdu._data_size
@@ -415,7 +426,7 @@ class TestCore(FitsTestCase):
         hdu.header['SIMPLE'] = False
         hdu.writeto(self.temp('test.fits'))
 
-        info = [(0, '', 'NonstandardHDU', 5, (), '', '')]
+        info = [(0, '', 1, 'NonstandardHDU', 5, (), '', '')]
         with fits.open(self.temp('test.fits')) as hdul:
             assert hdul.info(output=False) == info
             # NonstandardHDUs just treat the data as an unspecified array of
@@ -502,7 +513,7 @@ class TestCore(FitsTestCase):
             # Add a bunch of header keywords so that the data will be forced to
             # new offsets within the file:
             for idx in range(40):
-                hdul1[1].header['TEST%d' % idx] = 'test'
+                hdul1[1].header['TEST{}'.format(idx)] = 'test'
 
             hdul1.writeto(self.temp('test1.fits'))
             hdul1.writeto(self.temp('test2.fits'))
@@ -513,14 +524,14 @@ class TestCore(FitsTestCase):
             with fits.open(self.data('test0.fits')) as hdul2:
                 with fits.open(self.temp('test2.fits')) as hdul3:
                     for hdul in (hdul1, hdul3):
-                        for idx, hdus in enumerate(zip(hdul1, hdul)):
-                            hdu1, hdu2 = hdus
+                        for idx, hdus in enumerate(zip(hdul2, hdul)):
+                            hdu2, hdu = hdus
                             if idx != 1:
-                                assert hdu1.header == hdu2.header
+                                assert hdu.header == hdu2.header
                             else:
-                                assert (hdu1.header ==
-                                        hdu2.header[:len(hdu1.header)])
-                            assert np.all(hdu1.data == hdu2.data)
+                                assert (hdu2.header ==
+                                        hdu.header[:len(hdu2.header)])
+                            assert np.all(hdu.data == hdu2.data)
 
 
 class TestConvenienceFunctions(FitsTestCase):
@@ -529,33 +540,31 @@ class TestConvenienceFunctions(FitsTestCase):
         Simple test for writing a trivial header and some data to a file
         with the `writeto()` convenience function.
         """
-
+        filename = self.temp('array.fits')
         data = np.zeros((100, 100))
         header = fits.Header()
-        fits.writeto(self.temp('array.fits'), data, header=header,
-                     clobber=True)
-        hdul = fits.open(self.temp('array.fits'))
-        assert len(hdul) == 1
-        assert (data == hdul[0].data).all()
+        fits.writeto(filename, data, header=header, overwrite=True)
+        with fits.open(filename) as hdul:
+            assert len(hdul) == 1
+            assert (data == hdul[0].data).all()
 
-    @ignore_warnings(AstropyDeprecationWarning)
     def test_writeto_2(self):
         """
         Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/107
 
         Test of `writeto()` with a trivial header containing a single keyword.
         """
-
-        data = np.zeros((100,100))
+        filename = self.temp('array.fits')
+        data = np.zeros((100, 100))
         header = fits.Header()
         header.set('CRPIX1', 1.)
-        fits.writeto(self.temp('array.fits'), data, header=header,
-                     clobber=True, output_verify='silentfix')
-        hdul = fits.open(self.temp('array.fits'))
-        assert len(hdul) == 1
-        assert (data == hdul[0].data).all()
-        assert 'CRPIX1' in hdul[0].header
-        assert hdul[0].header['CRPIX1'] == 1.0
+        fits.writeto(filename, data, header=header,
+                     overwrite=True, output_verify='silentfix')
+        with fits.open(filename) as hdul:
+            assert len(hdul) == 1
+            assert (data == hdul[0].data).all()
+            assert 'CRPIX1' in hdul[0].header
+            assert hdul[0].header['CRPIX1'] == 1.0
 
 
 class TestFileFunctions(FitsTestCase):
@@ -573,8 +582,6 @@ class TestFileFunctions(FitsTestCase):
             fits.open(self.temp('foobar.fits'))
         except IOError as e:
             assert 'No such file or directory' in str(e)
-        except:
-            raise
 
         # But opening in ostream or append mode should be okay, since they
         # allow writing new files
@@ -607,7 +614,7 @@ class TestFileFunctions(FitsTestCase):
         # still how the original test case looked
         # Note: with statement not supported on GzipFile in older Python
         # versions
-        fileobj =  gzip.GzipFile(self.temp('test.fits.gz'), 'ab+')
+        fileobj = gzip.GzipFile(self.temp('test.fits.gz'), 'ab+')
         h = fits.PrimaryHDU()
         try:
             h.writeto(fileobj)
@@ -617,6 +624,37 @@ class TestFileFunctions(FitsTestCase):
         with fits.open(self.temp('test.fits.gz')) as hdul:
             assert hdul[0].header == h.header
 
+    def test_open_bzipped(self):
+        with ignore_warnings():
+            assert len(fits.open(self._make_bzip2_file())) == 5
+
+    def test_detect_bzipped(self):
+        """Test detection of a bzip2 file when the extension is not .bz2."""
+
+        with ignore_warnings():
+            assert len(fits.open(self._make_bzip2_file('test0.xx'))) == 5
+
+    def test_writeto_bzip2_fileobj(self):
+        """Test writing to a bz2.BZ2File file like object"""
+        fileobj = bz2.BZ2File(self.temp('test.fits.bz2'), 'w')
+        h = fits.PrimaryHDU()
+        try:
+            h.writeto(fileobj)
+        finally:
+            fileobj.close()
+
+        with fits.open(self.temp('test.fits.bz2')) as hdul:
+            assert hdul[0].header == h.header
+
+    def test_writeto_bzip2_filename(self):
+        """Test writing to a bzip2 file by name"""
+        filename = self.temp('testname.fits.bz2')
+        h = fits.PrimaryHDU()
+        h.writeto(filename)
+
+        with fits.open(self.temp('testname.fits.bz2')) as hdul:
+            assert hdul[0].header == h.header
+
     def test_open_zipped(self):
         zf = self._make_zip_file()
 
@@ -651,9 +689,6 @@ class TestFileFunctions(FitsTestCase):
         This tests reading from a ``GzipFile`` object from Astropy's
         compatibility copy of the ``gzip`` module.
         """
-
-        from ....utils.compat import gzip
-
         gf = gzip.GzipFile(self._make_gzip_file())
         try:
             assert len(fits.open(gf)) == 5
@@ -686,7 +721,8 @@ class TestFileFunctions(FitsTestCase):
 
         f = open(self.data('test0.fits'), 'rb')
         f.close()
-        assert len(fits.open(f)) == 5
+        with fits.open(f) as f2:
+            assert len(f2) == 5
 
     def test_read_open_gzip_file(self):
         """Read from an open gzip file object."""
@@ -703,10 +739,12 @@ class TestFileFunctions(FitsTestCase):
         gf = self._make_gzip_file()
         with fits.open(gf, mode='update') as h:
             h[0].header['EXPFLAG'] = 'ABNORMAL'
+            h[1].data[0, 0] = 1
         with fits.open(gf) as h:
             # Just to make sur ethe update worked; if updates work
             # normal writes should work too...
             assert h[0].header['EXPFLAG'] == 'ABNORMAL'
+            assert h[1].data[0, 0] == 1
 
     def test_write_read_gzip_file(self):
         """
@@ -758,7 +796,7 @@ class TestFileFunctions(FitsTestCase):
         assert old_mode == os.stat(filename).st_mode
 
     def test_fileobj_mode_guessing(self):
-        """Tests whether a file opened without a specified pyfits mode
+        """Tests whether a file opened without a specified io.fits mode
         ('readonly', etc.) is opened in a mode appropriate for the given file
         object.
         """
@@ -818,7 +856,7 @@ class TestFileFunctions(FitsTestCase):
         mmap.mmap = MockMmap
 
         # Force the mmap test to be rerun
-        _File._File__mmap_available = None
+        _File.__dict__['_mmap_available']._cache.clear()
 
         try:
             self.copy_file('test0.fits')
@@ -835,7 +873,7 @@ class TestFileFunctions(FitsTestCase):
                 assert h[1].data[0, 0] == 999
         finally:
             mmap.mmap = old_mmap
-            _File._File__mmap_available = None
+            _File.__dict__['_mmap_available']._cache.clear()
 
     def test_mmap_closing(self):
         """
@@ -908,7 +946,6 @@ class TestFileFunctions(FitsTestCase):
             def tell(self):
                 return self._foobar.tell()
 
-
         with open(self.data('test0.fits'), 'rb') as f:
             fileobj = MyFileLike(f)
 
@@ -978,6 +1015,63 @@ class TestFileFunctions(FitsTestCase):
         with fits.open(self.temp(filename)) as hdul:
             assert np.all(hdul[0].data == hdu.data)
 
+    def test_writeto_full_disk(self, monkeypatch):
+        """
+        Test that it gives a readable error when trying to write an hdulist
+        to a full disk.
+        """
+
+        def _writeto(self, array):
+            raise OSError("Fake error raised when writing file.")
+
+        def get_free_space_in_dir(path):
+            return 0
+
+        with pytest.raises(OSError) as exc:
+            monkeypatch.setattr(fits.hdu.base._BaseHDU, "_writeto", _writeto)
+            monkeypatch.setattr(data, "get_free_space_in_dir", get_free_space_in_dir)
+
+            n = np.arange(0, 1000, dtype='int64')
+            hdu = fits.PrimaryHDU(n)
+            hdulist = fits.HDUList(hdu)
+            filename = self.temp('test.fits')
+
+            with open(filename, mode='wb+') as fileobj:
+                hdulist.writeto(fileobj)
+
+        assert ("Not enough space on disk: requested 8000, available 0. "
+                "Fake error raised when writing file.") == exc.value.args[0]
+
+    def test_flush_full_disk(self, monkeypatch):
+        """
+        Test that it gives a readable error when trying to update an hdulist
+        to a full disk.
+        """
+        filename = self.temp('test.fits')
+        hdul = [fits.PrimaryHDU(), fits.ImageHDU()]
+        hdul = fits.HDUList(hdul)
+        hdul[0].data = np.arange(0, 1000, dtype='int64')
+        hdul.writeto(filename)
+
+        def _writedata(self, fileobj):
+            raise OSError("Fake error raised when writing file.")
+
+        def get_free_space_in_dir(path):
+            return 0
+
+        monkeypatch.setattr(fits.hdu.base._BaseHDU, "_writedata", _writedata)
+        monkeypatch.setattr(data, "get_free_space_in_dir",
+                            get_free_space_in_dir)
+
+        with pytest.raises(OSError) as exc:
+            with fits.open(filename, mode='update') as hdul:
+                hdul[0].data = np.arange(0, 1000, dtype='int64')
+                hdul.insert(1, fits.ImageHDU())
+                hdul.flush()
+
+        assert ("Not enough space on disk: requested 8000, available 0. "
+                "Fake error raised when writing file.") == exc.value.args[0]
+
     def _test_write_string_bytes_io(self, fileobj):
         """
         Implemented for both test_write_stringio and test_write_bytesio.
@@ -1004,6 +1098,15 @@ class TestFileFunctions(FitsTestCase):
 
         return zfile.filename
 
+    def _make_bzip2_file(self, filename='test0.fits.bz2'):
+        bzfile = self.temp(filename)
+        with open(self.data('test0.fits'), 'rb') as f:
+            bz = bz2.BZ2File(bzfile, 'w')
+            bz.write(f.read())
+            bz.close()
+
+        return bzfile
+
 
 class TestStreamingFunctions(FitsTestCase):
     """Test functionality of the StreamingHDU class."""
@@ -1012,6 +1115,7 @@ class TestStreamingFunctions(FitsTestCase):
         shdu = self._make_streaming_hdu(self.temp('new.fits'))
         assert isinstance(shdu.size, int)
         assert shdu.size == 100
+        shdu.close()
 
     @raises(ValueError)
     def test_streaming_hdu_file_wrong_mode(self):
@@ -1033,9 +1137,9 @@ class TestStreamingFunctions(FitsTestCase):
             shdu.write(arr)
             assert shdu.writecomplete
             assert shdu.size == 100
-        hdul = fits.open(self.temp('new.fits'))
-        assert len(hdul) == 1
-        assert (hdul[0].data == arr).all()
+        with fits.open(self.temp('new.fits')) as hdul:
+            assert len(hdul) == 1
+            assert (hdul[0].data == arr).all()
 
     def test_streaming_hdu_write_file_like(self):
         """Test streaming an HDU to an open file-like object."""
@@ -1068,14 +1172,19 @@ class TestStreamingFunctions(FitsTestCase):
         ihdu = fits.ImageHDU()
         ihdu.header['EXTNAME'] = 12345678
         hdul = fits.HDUList([phdu, ihdu])
+        filename = self.temp('temp.fits')
 
-        pytest.raises(fits.VerifyError, hdul.writeto, self.temp('temp.fits'),
+        pytest.raises(fits.VerifyError, hdul.writeto, filename,
                       output_verify='exception')
-        hdul.writeto(self.temp('temp.fits'), output_verify='fix')
-        with fits.open(self.temp('temp.fits')):
+        with pytest.warns(fits.verify.VerifyWarning,
+                          match='Verification reported errors'):
+            hdul.writeto(filename, output_verify='fix')
+        with fits.open(filename):
             assert hdul[1].name == '12345678'
             assert hdul[1].header['EXTNAME'] == '12345678'
 
+        hdul.close()
+
     def _make_streaming_hdu(self, fileobj):
         hd = fits.Header()
         hd['SIMPLE'] = (True, 'conforms to FITS standard')
@@ -1098,5 +1207,5 @@ class TestStreamingFunctions(FitsTestCase):
 
         # However, it should not fail if do_not_scale_image_data was used:
         # See https://github.com/astropy/astropy/issues/3766
-        hdul = fits.open(pth, memmap=True, do_not_scale_image_data=True)
-        hdul[0].data  # Just make sure it doesn't crash
+        with fits.open(pth, memmap=True, do_not_scale_image_data=True) as hdul:
+            hdul[0].data  # Just make sure it doesn't crash
