@@ -4,21 +4,32 @@ from __future__ import division, with_statement
 
 import math
 import os
+import platform
 import re
 import time
 import warnings
 
+import pytest
 import numpy as np
+from numpy.testing import assert_equal
 
+from ....extern.six.moves import range
 from ....io import fits
-from ....utils.exceptions import (AstropyDeprecationWarning,
-                                  AstropyPendingDeprecationWarning)
-from ....tests.helper import pytest, raises, catch_warnings, ignore_warnings
+from ....utils.exceptions import AstropyPendingDeprecationWarning
+from ....utils.compat import NUMPY_LT_1_12
+from ....tests.helper import raises, catch_warnings, ignore_warnings
 from ..hdu.compressed import SUBTRACTIVE_DITHER_1, DITHER_SEED_CHECKSUM
 from .test_table import comparerecords
 
 from . import FitsTestCase
 
+try:
+    import scipy  # pylint: disable=W0611
+except ImportError:
+    HAS_SCIPY = False
+else:
+    HAS_SCIPY = True
+
 
 class TestImageFunctions(FitsTestCase):
     def test_constructor_name_arg(self):
@@ -62,7 +73,6 @@ class TestImageFunctions(FitsTestCase):
         # Original header should be unchanged
         assert phdr['FILENAME'] == 'labq01i3q_rawtag.fits'
 
-    @raises(ValueError)
     def test_open(self):
         # The function "open" reads a FITS file into an HDUList object.  There
         # are three modes to open: "readonly" (the default), "append", and
@@ -75,13 +85,21 @@ class TestImageFunctions(FitsTestCase):
         # data parts are latent instantiation, so if we close the HDUList
         # without touching data, data can not be accessed.
         r.close()
-        r[1].data[:2, :2]
+
+        with pytest.raises(IndexError) as exc_info:
+            r[1].data[:2, :2]
+
+        # Check that the exception message is the enhanced version, not the
+        # default message from list.__getitem__
+        assert str(exc_info.value) == ('HDU not found, possibly because the index '
+                                       'is out of range, or because the file was '
+                                       'closed before all HDUs were read')
 
     def test_open_2(self):
         r = fits.open(self.data('test0.fits'))
 
-        info = ([(0, 'PRIMARY', 'PrimaryHDU', 138, (), '', '')] +
-                [(x, 'SCI', 'ImageHDU', 61, (40, 40), 'int16', '')
+        info = ([(0, 'PRIMARY', 1, 'PrimaryHDU', 138, (), '', '')] +
+                [(x, 'SCI', x, 'ImageHDU', 61, (40, 40), 'int16', '')
                  for x in range(1, 5)])
 
         try:
@@ -89,6 +107,40 @@ class TestImageFunctions(FitsTestCase):
         finally:
             r.close()
 
+    def test_open_3(self):
+        # Test that HDUs cannot be accessed after the file was closed
+        r = fits.open(self.data('test0.fits'))
+        r.close()
+        with pytest.raises(IndexError) as exc_info:
+            r[1]
+
+        # Check that the exception message is the enhanced version, not the
+        # default message from list.__getitem__
+        assert str(exc_info.value) == ('HDU not found, possibly because the index '
+                                       'is out of range, or because the file was '
+                                       'closed before all HDUs were read')
+
+        # Test that HDUs can be accessed with lazy_load_hdus=False
+        r = fits.open(self.data('test0.fits'), lazy_load_hdus=False)
+        r.close()
+        assert isinstance(r[1], fits.ImageHDU)
+        assert len(r) == 5
+
+        with pytest.raises(IndexError) as exc_info:
+            r[6]
+        assert str(exc_info.value) == 'list index out of range'
+
+        # And the same with the global config item
+        assert fits.conf.lazy_load_hdus  # True by default
+        fits.conf.lazy_load_hdus = False
+        try:
+            r = fits.open(self.data('test0.fits'))
+            r.close()
+            assert isinstance(r[1], fits.ImageHDU)
+            assert len(r) == 5
+        finally:
+            fits.conf.lazy_load_hdus = True
+
     def test_primary_with_extname(self):
         """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/151
 
@@ -104,7 +156,7 @@ class TestImageFunctions(FitsTestCase):
         assert hdul[0].name == 'XPRIMARY'
         assert hdul[0].name == hdul[0].header['EXTNAME']
 
-        info = [(0, 'XPRIMARY', 'PrimaryHDU', 5, (), '', '')]
+        info = [(0, 'XPRIMARY', 1, 'PrimaryHDU', 5, (), '', '')]
         assert hdul.info(output=False) == info
 
         assert hdul['PRIMARY'] is hdul['XPRIMARY']
@@ -117,11 +169,9 @@ class TestImageFunctions(FitsTestCase):
         with fits.open(self.temp('test.fits')) as hdul:
             assert hdul[0].name == 'XPRIMARY2'
 
-    @ignore_warnings(AstropyDeprecationWarning)
+    @pytest.mark.xfail(platform.system() == 'Windows' and not NUMPY_LT_1_12,
+                              reason='https://github.com/astropy/astropy/issues/5797')
     def test_io_manipulation(self):
-        # This legacy test also tests numerous deprecated interfaces for
-        # backwards compatibility
-
         # Get a keyword value.  An extension can be referred by name or by
         # number.  Both extension and keyword names are case insensitive.
         with fits.open(self.data('test0.fits')) as r:
@@ -136,19 +186,19 @@ class TestImageFunctions(FitsTestCase):
             # append (using "update()") a new card
             r[0].header['xxx'] = 1.234e56
 
-            assert (str(r[0].header.ascard[-3:]) ==
+            assert ('\n'.join(str(x) for x in r[0].header.cards[-3:]) ==
                     "EXPFLAG = 'NORMAL            ' / Exposure interruption indicator                \n"
                     "FILENAME= 'vtest3.fits'        / File name                                      \n"
                     "XXX     =            1.234E+56                                                  ")
 
             # rename a keyword
-            r[0].header.rename_key('filename', 'fname')
-            pytest.raises(ValueError, r[0].header.rename_key, 'fname',
+            r[0].header.rename_keyword('filename', 'fname')
+            pytest.raises(ValueError, r[0].header.rename_keyword, 'fname',
                           'history')
 
-            pytest.raises(ValueError, r[0].header.rename_key, 'fname',
+            pytest.raises(ValueError, r[0].header.rename_keyword, 'fname',
                           'simple')
-            r[0].header.rename_key('fname', 'filename')
+            r[0].header.rename_keyword('fname', 'filename')
 
             # get a subsection of data
             assert np.array_equal(r[2].data[:3, :3],
@@ -241,9 +291,9 @@ class TestImageFunctions(FitsTestCase):
         data = np.ones((3, 5), dtype=np.float32)
         hdu = fits.ImageHDU(data=data, name='SCI')
         assert np.array_equal(hdu.data,
-                              np.array([[1.,  1.,  1.,  1.,  1.],
-                                        [1.,  1.,  1.,  1.,  1.],
-                                        [1.,  1.,  1.,  1.,  1.]],
+                              np.array([[1., 1., 1., 1., 1.],
+                                        [1., 1., 1., 1., 1.],
+                                        [1., 1., 1., 1., 1.]],
                                        dtype=np.float32))
 
         # create an HDU with header and data
@@ -252,7 +302,7 @@ class TestImageFunctions(FitsTestCase):
         hdu2 = fits.ImageHDU(header=r[1].header, data=np.array([1, 2],
                              dtype='int32'))
 
-        assert (str(hdu2.header.ascard[1:5]) ==
+        assert ('\n'.join(str(x) for x in hdu2.header.cards[1:5]) ==
             "BITPIX  =                   32 / array data type                                \n"
             "NAXIS   =                    1 / number of array dimensions                     \n"
             "NAXIS1  =                    2                                                  \n"
@@ -321,8 +371,8 @@ class TestImageFunctions(FitsTestCase):
                                          [572, 573, 574]]]))
 
         assert np.array_equal(fs[0].section[:, :, :][:3, :2, :2],
-                              np.array([[[0,   1],
-                                         [11,  12]],
+                              np.array([[[0, 1],
+                                         [11, 12]],
                                         [[110, 111],
                                          [121, 122]],
                                         [[220, 221],
@@ -336,9 +386,10 @@ class TestImageFunctions(FitsTestCase):
         assert np.array_equal(fs[0].section[:, ::2], dat[:, ::2])
         assert np.array_equal(fs[0].section[:, [1, 2, 4], 3],
                               dat[:, [1, 2, 4], 3])
-        assert np.array_equal(
-            fs[0].section[:, np.array([True, False, True]), :],
-            dat[:, np.array([True, False, True]), :])
+        bool_index = np.array([True, False, True, True, False,
+                               False, True, True, False, True])
+        assert np.array_equal(fs[0].section[:, bool_index, :],
+                              dat[:, bool_index, :])
 
         assert np.array_equal(
             fs[0].section[3:6, 3, :, ...], dat[3:6, 3, :, ...])
@@ -590,6 +641,16 @@ class TestImageFunctions(FitsTestCase):
         f = fits.open(self.temp('test_new.fits'), uint=True)
         assert f[1].data.dtype == 'uint16'
 
+    def test_scale_with_explicit_bzero_bscale(self):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/6399
+        """
+        hdu1 = fits.PrimaryHDU()
+        hdu2 = fits.ImageHDU(np.random.rand(100,100))
+        # The line below raised an exception in astropy 2.0, so if it does not
+        # raise an error here, that is progress.
+        hdu2.scale(type='uint8', bscale=1, bzero=0)
+
     def test_uint_header_consistency(self):
         """
         Regression test for https://github.com/astropy/astropy/issues/2305
@@ -606,24 +667,24 @@ class TestImageFunctions(FitsTestCase):
                 # Otherwise may get an overflow error, at least on Python 2
                 max_uint = np.uint64(int_size)
 
-            dtype = 'uint%d' % int_size
+            dtype = 'uint{}'.format(int_size)
             arr = np.empty(100, dtype=dtype)
             arr.fill(max_uint)
             arr -= np.arange(100, dtype=dtype)
 
             uint_hdu = fits.PrimaryHDU(data=arr)
             assert np.all(uint_hdu.data == arr)
-            assert uint_hdu.data.dtype.name == 'uint%d' % int_size
+            assert uint_hdu.data.dtype.name == 'uint{}'.format(int_size)
             assert 'BZERO' in uint_hdu.header
             assert uint_hdu.header['BZERO'] == (2 ** (int_size - 1))
 
-            filename = 'uint%d.fits' % int_size
+            filename = 'uint{}.fits'.format(int_size)
             uint_hdu.writeto(self.temp(filename))
 
             with fits.open(self.temp(filename), uint=True) as hdul:
                 new_uint_hdu = hdul[0]
                 assert np.all(new_uint_hdu.data == arr)
-                assert new_uint_hdu.data.dtype.name == 'uint%d' % int_size
+                assert new_uint_hdu.data.dtype.name == 'uint{}'.format(int_size)
                 assert 'BZERO' in new_uint_hdu.header
                 assert new_uint_hdu.header['BZERO'] == (2 ** (int_size - 1))
 
@@ -651,12 +712,11 @@ class TestImageFunctions(FitsTestCase):
             tmp_uint = fits.PrimaryHDU(arr)
             filename = 'unsigned_int.fits'
             tmp_uint.writeto(self.temp(filename))
-            f = fits.open(self.temp(filename),
-                          do_not_scale_image_data=do_not_scale)
-
-            uint_hdu = f[0]
-            # Force a read before we close.
-            _ = uint_hdu.data
+            with fits.open(self.temp(filename),
+                           do_not_scale_image_data=do_not_scale) as f:
+                uint_hdu = f[0]
+                # Force a read before we close.
+                _ = uint_hdu.data
         else:
             uint_hdu = fits.PrimaryHDU(arr,
                                        do_not_scale_image_data=do_not_scale)
@@ -753,14 +813,17 @@ class TestImageFunctions(FitsTestCase):
 
         # Let's just add a value to the data that should be converted to NaN
         # when it is read back in:
+        filename = self.temp('test.fits')
         hdu.data[0] = 9999
         hdu.header['BLANK'] = 9999
-        hdu.writeto(self.temp('test.fits'))
+        hdu.writeto(filename)
 
-        with fits.open(self.temp('test.fits')) as hdul:
+        with fits.open(filename) as hdul:
             data = hdul[0].data
             assert np.isnan(data[0])
-            hdul.writeto(self.temp('test2.fits'))
+            with pytest.warns(fits.verify.VerifyWarning,
+                              match="Invalid 'BLANK' keyword in header"):
+                hdul.writeto(self.temp('test2.fits'))
 
         # Now reopen the newly written file.  It should not have a 'BLANK'
         # keyword
@@ -772,12 +835,12 @@ class TestImageFunctions(FitsTestCase):
                 assert np.isnan(data[0])
 
         # Finally, test that scale_back keeps the BLANKs correctly
-        with fits.open(self.temp('test.fits'), scale_back=True,
+        with fits.open(filename, scale_back=True,
                        mode='update') as hdul3:
             data = hdul3[0].data
             assert np.isnan(data[0])
 
-        with fits.open(self.temp('test.fits'),
+        with fits.open(filename,
                        do_not_scale_image_data=True) as hdul4:
             assert hdul4[0].header['BLANK'] == 9999
             assert hdul4[0].header['BSCALE'] == 1.23
@@ -805,7 +868,7 @@ class TestImageFunctions(FitsTestCase):
         hdul = fits.open(self.data('fixed-1890.fits'))
         orig_data = hdul[0].data
         with ignore_warnings():
-            hdul.writeto(self.temp('test_new.fits'), clobber=True)
+            hdul.writeto(self.temp('test_new.fits'), overwrite=True)
         hdul.close()
         hdul = fits.open(self.temp('test_new.fits'))
         assert (hdul[0].data == orig_data).all()
@@ -816,7 +879,7 @@ class TestImageFunctions(FitsTestCase):
         # https://aeon.stsci.edu/ssb/trac/pyfits/ticket/84
         hdul = fits.open(self.data('fixed-1890.fits'))
         with ignore_warnings():
-            hdul.writeto(self.temp('test_new.fits'), clobber=True)
+            hdul.writeto(self.temp('test_new.fits'), overwrite=True)
         hdul.close()
         hdul = fits.open(self.temp('test_new.fits'))
         assert (hdul[0].data == orig_data).all()
@@ -825,7 +888,7 @@ class TestImageFunctions(FitsTestCase):
         # Test opening/closing/reopening a scaled file in update mode
         hdul = fits.open(self.data('fixed-1890.fits'),
                          do_not_scale_image_data=True)
-        hdul.writeto(self.temp('test_new.fits'), clobber=True,
+        hdul.writeto(self.temp('test_new.fits'), overwrite=True,
                      output_verify='silentfix')
         hdul.close()
         hdul = fits.open(self.temp('test_new.fits'))
@@ -962,7 +1025,9 @@ class TestImageFunctions(FitsTestCase):
         data = np.arange(100, dtype=np.float64)
         hdu = fits.PrimaryHDU(data)
         hdu.header['BLANK'] = 'nan'
-        hdu.writeto(self.temp('test.fits'))
+        with pytest.warns(fits.verify.VerifyWarning, match="Invalid value for "
+                          "'BLANK' keyword in header: 'nan'"):
+            hdu.writeto(self.temp('test.fits'))
 
         with catch_warnings() as w:
             with fits.open(self.temp('test.fits')) as hdul:
@@ -992,6 +1057,45 @@ class TestImageFunctions(FitsTestCase):
         hdul = fits.HDUList.fromstring(file_data)
         assert np.allclose(hdul[0].data, a)
 
+    def test_set_data(self):
+        """
+        Test data assignment - issue #5087
+        """
+
+        im = fits.ImageHDU()
+        ar = np.arange(12)
+        im.data = ar
+
+    def test_scale_bzero_with_int_data(self):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/4600
+        """
+
+        a = np.arange(100, 200, dtype=np.int16)
+
+        hdu1 = fits.PrimaryHDU(data=a.copy())
+        hdu2 = fits.PrimaryHDU(data=a.copy())
+        # Previously the following line would throw a TypeError,
+        # now it should be identical to the integer bzero case
+        hdu1.scale('int16', bzero=99.0)
+        hdu2.scale('int16', bzero=99)
+        assert np.allclose(hdu1.data, hdu2.data)
+
+    def test_scale_back_uint_assignment(self):
+        """
+        Extend fix for #4600 to assignment to data
+
+        Suggested by:
+        https://github.com/astropy/astropy/pull/4602#issuecomment-208713748
+        """
+
+        a = np.arange(100, 200, dtype=np.uint16)
+        fits.PrimaryHDU(a).writeto(self.temp('test.fits'))
+        with fits.open(self.temp('test.fits'), mode="update",
+                       scale_back=True) as (hdu,):
+            hdu.data[:] = 0
+            assert np.allclose(hdu.data, 0)
+
 
 class TestCompressedImage(FitsTestCase):
     def test_empty(self):
@@ -1021,6 +1125,7 @@ class TestCompressedImage(FitsTestCase):
         ('data', 'compression_type', 'quantize_level', 'byte_order'),
         sum([[(np.zeros((2, 10, 10), dtype=np.float32), 'RICE_1', 16, bo),
               (np.zeros((2, 10, 10), dtype=np.float32), 'GZIP_1', -0.01, bo),
+              (np.zeros((2, 10, 10), dtype=np.float32), 'GZIP_2', -0.01, bo),
               (np.zeros((100, 100)) + 1, 'HCOMPRESS_1', 16, bo)]
              for bo in ('<', '>')], []))
     def test_comp_image(self, data, compression_type, quantize_level,
@@ -1032,7 +1137,7 @@ class TestCompressedImage(FitsTestCase):
                                  compressionType=compression_type,
                                  quantizeLevel=quantize_level)
         ofd.append(chdu)
-        ofd.writeto(self.temp('test_new.fits'), clobber=True)
+        ofd.writeto(self.temp('test_new.fits'), overwrite=True)
         ofd.close()
         with fits.open(self.temp('test_new.fits')) as fd:
             assert (fd[1].data == data).all()
@@ -1041,6 +1146,36 @@ class TestCompressedImage(FitsTestCase):
             assert fd[1].header['NAXIS2'] == chdu.header['NAXIS2']
             assert fd[1].header['BITPIX'] == chdu.header['BITPIX']
 
+    @pytest.mark.skipif('not HAS_SCIPY')
+    def test_comp_image_quantize_level(self):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/5969
+
+        Test that quantize_level is used.
+
+        """
+        import scipy.misc
+        np.random.seed(42)
+        data = scipy.misc.ascent() + np.random.randn(512, 512)*10
+
+        fits.ImageHDU(data).writeto(self.temp('im1.fits'))
+        fits.CompImageHDU(data, compression_type='RICE_1', quantize_method=1,
+                          quantize_level=-1, dither_seed=5)\
+            .writeto(self.temp('im2.fits'))
+        fits.CompImageHDU(data, compression_type='RICE_1', quantize_method=1,
+                          quantize_level=-100, dither_seed=5)\
+            .writeto(self.temp('im3.fits'))
+
+        im1 = fits.getdata(self.temp('im1.fits'))
+        im2 = fits.getdata(self.temp('im2.fits'))
+        im3 = fits.getdata(self.temp('im3.fits'))
+
+        assert not np.array_equal(im2, im3)
+        assert np.isclose(np.min(im1 - im2), -0.5, atol=1e-3)
+        assert np.isclose(np.max(im1 - im2), 0.5, atol=1e-3)
+        assert np.isclose(np.min(im1 - im3), -50, atol=1e-1)
+        assert np.isclose(np.max(im1 - im3), 50, atol=1e-1)
+
     @ignore_warnings(AstropyPendingDeprecationWarning)
     def test_comp_image_hcompression_1_invalid_data(self):
         """
@@ -1070,7 +1205,9 @@ class TestCompressedImage(FitsTestCase):
         hdu.writeto(self.temp('test.fits'))
 
         with fits.open(self.temp('test.fits')) as hdul:
-            assert (hdul['SCI'].data == cube).all()
+            # HCOMPRESSed images are allowed to deviate from the original by
+            # about 1/quantize_level of the RMS in each tile.
+            assert np.abs(hdul['SCI'].data - cube).max() < 1./15.
 
     def test_subtractive_dither_seed(self):
         """
@@ -1211,7 +1348,7 @@ class TestCompressedImage(FitsTestCase):
         hdul = fits.open(self.temp('fixed-1890-z.fits'))
         orig_data = hdul[1].data
         with ignore_warnings():
-            hdul.writeto(self.temp('test_new.fits'), clobber=True)
+            hdul.writeto(self.temp('test_new.fits'), overwrite=True)
         hdul.close()
         hdul = fits.open(self.temp('test_new.fits'))
         assert (hdul[1].data == orig_data).all()
@@ -1222,7 +1359,7 @@ class TestCompressedImage(FitsTestCase):
         # https://aeon.stsci.edu/ssb/trac/pyfits/ticket/84
         hdul = fits.open(self.temp('fixed-1890-z.fits'))
         with ignore_warnings():
-            hdul.writeto(self.temp('test_new.fits'), clobber=True)
+            hdul.writeto(self.temp('test_new.fits'), overwrite=True)
         hdul.close()
         hdul = fits.open(self.temp('test_new.fits'))
         assert (hdul[1].data == orig_data).all()
@@ -1231,7 +1368,7 @@ class TestCompressedImage(FitsTestCase):
         # Test opening/closing/reopening a scaled file in update mode
         hdul = fits.open(self.temp('fixed-1890-z.fits'),
                          do_not_scale_image_data=True)
-        hdul.writeto(self.temp('test_new.fits'), clobber=True,
+        hdul.writeto(self.temp('test_new.fits'), overwrite=True,
                      output_verify='silentfix')
         hdul.close()
         hdul = fits.open(self.temp('test_new.fits'))
@@ -1304,7 +1441,7 @@ class TestCompressedImage(FitsTestCase):
         chdu2 = fits.CompImageHDU(data=noise, compressionType='GZIP_1',
                                   quantizeLevel=0.0)  # No quantization
         with ignore_warnings():
-            chdu2.writeto(self.temp('test.fits'), clobber=True)
+            chdu2.writeto(self.temp('test.fits'), overwrite=True)
 
         with fits.open(self.temp('test.fits')) as h:
             assert (noise == h[1].data).all()
@@ -1343,7 +1480,7 @@ class TestCompressedImage(FitsTestCase):
             assert 'test1' in hdul[1].header
             assert hdul[1].header['test1'] == 'test'
             assert 'test2' in hdul[1].header
-            assert  hdul[1].header['test2'] == 'test2'
+            assert hdul[1].header['test2'] == 'test2'
 
         # Test update via index now:
         with fits.open(self.temp('comp.fits'), mode='update') as hdul:
@@ -1402,7 +1539,7 @@ class TestCompressedImage(FitsTestCase):
                 hdr[keyword] = value
                 assert len(w) == 1
                 assert str(w[0].message).startswith(
-                        "Keyword %r is reserved" % keyword)
+                        "Keyword {!r} is reserved".format(keyword))
                 assert keyword not in hdr
 
         with fits.open(self.data('comp.fits')) as hdul:
@@ -1434,6 +1571,20 @@ class TestCompressedImage(FitsTestCase):
             assert 'ZHECKSUM' in tblhdr
             assert tblhdr['ZHECKSUM'] == 'abcd1234'
 
+    def test_compression_header_append2(self):
+        """
+        Regresion test for issue https://github.com/astropy/astropy/issues/5827
+        """
+        with fits.open(self.data('comp.fits')) as hdul:
+            header = hdul[1].header
+            while (len(header) < 1000):
+                header.append()    # pad with grow room
+
+            # Append stats to header:
+            header.append(("Q1_OSAVG", 1, "[adu] quadrant 1 overscan mean"))
+            header.append(("Q1_OSSTD", 1, "[adu] quadrant 1 overscan stddev"))
+            header.append(("Q1_OSMED", 1, "[adu] quadrant 1 overscan median"))
+
     def test_compression_header_insert(self):
         with fits.open(self.data('comp.fits')) as hdul:
             imghdr = hdul[1].header
@@ -1554,11 +1705,181 @@ class TestCompressedImage(FitsTestCase):
             # technically it isn't invalid either :/
             assert hdul[1]._header.count('ZTENSION') == 2
 
+    def test_scale_bzero_with_compressed_int_data(self):
+        """
+        Regression test for https://github.com/astropy/astropy/issues/4600
+        and https://github.com/astropy/astropy/issues/4588
 
-def test_set_data():
+        Identical to test_scale_bzero_with_int_data() but uses a compressed
+        image.
+        """
+
+        a = np.arange(100, 200, dtype=np.int16)
+
+        hdu1 = fits.CompImageHDU(data=a.copy())
+        hdu2 = fits.CompImageHDU(data=a.copy())
+        # Previously the following line would throw a TypeError,
+        # now it should be identical to the integer bzero case
+        hdu1.scale('int16', bzero=99.0)
+        hdu2.scale('int16', bzero=99)
+        assert np.allclose(hdu1.data, hdu2.data)
+
+    def test_scale_back_compressed_uint_assignment(self):
+        """
+        Extend fix for #4600 to assignment to data
+
+        Identical to test_scale_back_uint_assignment() but uses a compressed
+        image.
+
+        Suggested by:
+        https://github.com/astropy/astropy/pull/4602#issuecomment-208713748
+        """
+
+        a = np.arange(100, 200, dtype=np.uint16)
+        fits.CompImageHDU(a).writeto(self.temp('test.fits'))
+        with fits.open(self.temp('test.fits'), mode="update",
+                       scale_back=True) as hdul:
+            hdul[1].data[:] = 0
+            assert np.allclose(hdul[1].data, 0)
+
+    def test_compressed_header_missing_znaxis(self):
+        a = np.arange(100, 200, dtype=np.uint16)
+        comp_hdu = fits.CompImageHDU(a)
+        comp_hdu._header.pop('ZNAXIS')
+        with pytest.raises(TypeError):
+            comp_hdu.compressed_data
+        comp_hdu = fits.CompImageHDU(a)
+        comp_hdu._header.pop('ZBITPIX')
+        with pytest.raises(TypeError):
+            comp_hdu.compressed_data
+
+    @pytest.mark.parametrize(
+        ('keyword', 'dtype', 'expected'),
+        [('BSCALE', np.uint8, np.float32), ('BSCALE', np.int16, np.float32),
+         ('BSCALE', np.int32, np.float64), ('BZERO', np.uint8, np.float32),
+         ('BZERO', np.int16, np.float32), ('BZERO', np.int32, np.float64)])
+    def test_compressed_scaled_float(self, keyword, dtype, expected):
+        """
+        If BSCALE,BZERO is set to floating point values, the image
+        should be floating-point.
+
+        https://github.com/astropy/astropy/pull/6492
+
+        Parameters
+        ----------
+        keyword : `str`
+            Keyword to set to a floating-point value to trigger
+            floating-point pixels.
+        dtype : `numpy.dtype`
+            Type of original array.
+        expected : `numpy.dtype`
+            Expected type of uncompressed array.
+        """
+        value = 1.23345  # A floating-point value
+        hdu = fits.CompImageHDU(np.arange(0, 10, dtype=dtype))
+        hdu.header[keyword] = value
+        hdu.writeto(self.temp('test.fits'))
+        del hdu
+        with fits.open(self.temp('test.fits')) as hdu:
+            assert hdu[1].header[keyword] == value
+            assert hdu[1].data.dtype == expected
+
+
+def test_comphdu_bscale(tmpdir):
     """
-    Test data assignment - issue #5087
+    Regression test for a bug that caused extensions that used BZERO and BSCALE
+    that got turned into CompImageHDU to end up with BZERO/BSCALE before the
+    TFIELDS.
     """
-    im = fits.ImageHDU()
-    ar = np.arange(12)
-    im.data = ar
+
+    filename1 = tmpdir.join('3hdus.fits').strpath
+    filename2 = tmpdir.join('3hdus_comp.fits').strpath
+
+    x = np.random.random((100, 100))*100
+
+    x0 = fits.PrimaryHDU()
+    x1 = fits.ImageHDU(np.array(x-50, dtype=int), uint=True)
+    x1.header['BZERO'] = 20331
+    x1.header['BSCALE'] = 2.3
+    hdus = fits.HDUList([x0, x1])
+    hdus.writeto(filename1)
+
+    # fitsverify (based on cfitsio) should fail on this file, only seeing the
+    # first HDU.
+    hdus = fits.open(filename1)
+    hdus[1] = fits.CompImageHDU(data=hdus[1].data.astype(np.uint32),
+                                header=hdus[1].header)
+    hdus.writeto(filename2)
+
+    # open again and verify
+    hdus = fits.open(filename2)
+    hdus[1].verify('exception')
+
+
+def test_scale_implicit_casting():
+
+    # Regression test for an issue that occurred because Numpy now does not
+    # allow implicit type casting during inplace operations.
+
+    hdu = fits.ImageHDU(np.array([1], dtype=np.int32))
+    hdu.scale(bzero=1.3)
+
+
+def test_bzero_implicit_casting_compressed():
+
+    # Regression test for an issue that occurred because Numpy now does not
+    # allow implicit type casting during inplace operations. Astropy is
+    # actually not able to produce a file that triggers the failure - the
+    # issue occurs when using unsigned integer types in the FITS file, in which
+    # case BZERO should be 32768. But if the keyword is stored as 32768.0, then
+    # it was possible to trigger the implicit casting error.
+
+    filename = os.path.join(os.path.dirname(__file__),
+                            'data', 'compressed_float_bzero.fits')
+
+    hdu = fits.open(filename)[1]
+    hdu.data
+
+
+def test_bzero_mishandled_info(tmpdir):
+    # Regression test for #5507:
+    # Calling HDUList.info() on a dataset which applies a zeropoint
+    # from BZERO but which astropy.io.fits does not think it needs
+    # to resize to a new dtype results in an AttributeError.
+    filename = tmpdir.join('floatimg_with_bzero.fits').strpath
+    hdu = fits.ImageHDU(np.zeros((10, 10)))
+    hdu.header['BZERO'] = 10
+    hdu.writeto(filename, overwrite=True)
+    hdul = fits.open(filename)
+    hdul.info()
+
+
+def test_image_write_readonly(tmpdir):
+
+    # Regression test to make sure that we can write out read-only arrays (#5512)
+
+    x = np.array([1, 2, 3])
+    x.setflags(write=False)
+    ghdu = fits.ImageHDU(data=x)
+    ghdu.add_datasum()
+
+    filename = tmpdir.join('test.fits').strpath
+
+    ghdu.writeto(filename)
+
+    with fits.open(filename) as hdulist:
+        assert_equal(hdulist[1].data, [1, 2, 3])
+
+    # Same for compressed HDU
+    x = np.array([1.0, 2.0, 3.0])
+    x.setflags(write=False)
+    ghdu = fits.CompImageHDU(data=x)
+    # add_datasum does not work for CompImageHDU
+    # ghdu.add_datasum()
+
+    filename = tmpdir.join('test2.fits').strpath
+
+    ghdu.writeto(filename)
+
+    with fits.open(filename) as hdulist:
+        assert_equal(hdulist[1].data, [1.0, 2.0, 3.0])
