@@ -2,10 +2,11 @@
 
 from __future__ import division
 
-import gzip as _system_gzip
+import gzip
 import itertools
 import io
 import mmap
+import operator
 import os
 import platform
 import signal
@@ -16,6 +17,8 @@ import textwrap
 import threading
 import warnings
 import weakref
+from contextlib import contextmanager
+from ...utils import data
 
 from distutils.version import LooseVersion
 
@@ -32,20 +35,17 @@ except ImportError:
 from ...extern import six
 from ...extern.six import (string_types, integer_types, text_type,
                            binary_type, next)
-from ...extern.six.moves import zip
+from ...extern.six.moves import zip, range
 from ...utils import wraps
-from ...utils.compat import ignored
-from ...utils.compat import gzip as _astropy_gzip
+from ...utils.compat import suppress
 from ...utils.exceptions import AstropyUserWarning
 
-
-_GZIP_FILE_TYPES = (_astropy_gzip.GzipFile, _system_gzip.GzipFile)
-
-
-if six.PY3:
-    cmp = lambda a, b: (a > b) - (a < b)
-elif six.PY2:
+if six.PY2:
     cmp = cmp
+else:
+    cmp = lambda a, b: (a > b) - (a < b)
+
+all_integer_types = integer_types + (np.integer,)
 
 
 class NotifierMixin(object):
@@ -117,7 +117,7 @@ class NotifierMixin(object):
         if self._listeners is None:
             return
 
-        with ignored(KeyError):
+        with suppress(KeyError):
             del self._listeners[id(listener)]
 
     def _notify(self, notification, *args, **kwargs):
@@ -206,14 +206,14 @@ def itersubclasses(cls, _seen=None):
 
     if not isinstance(cls, type):
         raise TypeError('itersubclasses must be called with '
-                        'new-style classes, not %.100r' % cls)
+                        'new-style classes, not {!r}'.format(cls))
     if _seen is None:
         _seen = set()
     try:
         subs = cls.__subclasses__()
     except TypeError:  # fails only when cls is type
         subs = cls.__subclasses__(cls)
-    for sub in sorted(subs, key=lambda s: s.__name__):
+    for sub in sorted(subs, key=operator.attrgetter('__name__')):
         if sub not in _seen:
             _seen.add(sub)
             yield sub
@@ -240,8 +240,9 @@ def ignore_sigint(func):
                 self.sigint_received = False
 
             def __call__(self, signum, frame):
-                warnings.warn('KeyboardInterrupt ignored until %s is '
-                              'complete!' % func.__name__, AstropyUserWarning)
+                warnings.warn('KeyboardInterrupt ignored until {} is '
+                              'complete!'.format(func.__name__),
+                              AstropyUserWarning)
                 self.sigint_received = True
 
         sigint_handler = SigintHandler()
@@ -266,12 +267,6 @@ def ignore_sigint(func):
     return wrapped
 
 
-def first(iterable):
-    """Returns the first element from an iterable."""
-
-    return next(iter(iterable))
-
-
 def pairwise(iterable):
     """Return the items of an iterable paired with its next item.
 
@@ -312,7 +307,7 @@ def isreadable(f):
     sense approximation of io.IOBase.readable.
     """
 
-    if six.PY3 and hasattr(f, 'readable'):
+    if not six.PY2 and hasattr(f, 'readable'):
         return f.readable()
 
     if hasattr(f, 'closed') and f.closed:
@@ -336,7 +331,7 @@ def iswritable(f):
     sense approximation of io.IOBase.writable.
     """
 
-    if six.PY3 and hasattr(f, 'writable'):
+    if not six.PY2 and hasattr(f, 'writable'):
         return f.writable()
 
     if hasattr(f, 'closed') and f.closed:
@@ -354,7 +349,18 @@ def iswritable(f):
     return True
 
 
-if six.PY3:
+if six.PY2:
+    def isfile(f):
+        """
+        Returns True if the given object represents an OS-level file (that is,
+        ``isinstance(f, file)``).
+
+        On Python 3 this also returns True if the given object is higher level
+        wrapper on top of a FileIO object, such as a TextIOWrapper.
+        """
+
+        return isinstance(f, file)
+else:
     def isfile(f):
         """
         Returns True if the given object represents an OS-level file (that is,
@@ -371,34 +377,9 @@ if six.PY3:
         elif hasattr(f, 'raw'):
             return isfile(f.raw)
         return False
-elif six.PY2:
-    def isfile(f):
-        """
-        Returns True if the given object represents an OS-level file (that is,
-        ``isinstance(f, file)``).
-
-        On Python 3 this also returns True if the given object is higher level
-        wrapper on top of a FileIO object, such as a TextIOWrapper.
-        """
-
-        return isinstance(f, file)
 
 
-if six.PY3:
-    def fileobj_open(filename, mode):
-        """
-        A wrapper around the `open()` builtin.
-
-        This exists because in Python 3, `open()` returns an
-        `io.BufferedReader` by default.  This is bad, because
-        `io.BufferedReader` doesn't support random access, which we need in
-        some cases.  In the Python 3 case (implemented in the py3compat module)
-        we must call open with buffering=0 to get a raw random-access file
-        reader.
-        """
-
-        return open(filename, mode, buffering=0)
-elif six.PY2:
+if six.PY2:
     def fileobj_open(filename, mode):
         """
         A wrapper around the `open()` builtin.
@@ -412,6 +393,20 @@ elif six.PY2:
         """
 
         return open(filename, mode)
+else:
+    def fileobj_open(filename, mode):
+        """
+        A wrapper around the `open()` builtin.
+
+        This exists because in Python 3, `open()` returns an
+        `io.BufferedReader` by default.  This is bad, because
+        `io.BufferedReader` doesn't support random access, which we need in
+        some cases.  In the Python 3 case (implemented in the py3compat module)
+        we must call open with buffering=0 to get a raw random-access file
+        reader.
+        """
+
+        return open(filename, mode, buffering=0)
 
 
 def fileobj_name(f):
@@ -423,7 +418,7 @@ def fileobj_name(f):
 
     if isinstance(f, string_types):
         return f
-    elif isinstance(f, _GZIP_FILE_TYPES):
+    elif isinstance(f, gzip.GzipFile):
         # The .name attribute on GzipFiles does not always represent the name
         # of the file being read/written--it can also represent the original
         # name of the file being compressed
@@ -444,10 +439,16 @@ def fileobj_name(f):
 
 def fileobj_closed(f):
     """
-    Returns True if the given file-like object is closed or if f is not a
-    file-like object.
+    Returns True if the given file-like object is closed or if f is a string
+    (and assumed to be a pathname).
+
+    Returns False for all other types of objects, under the assumption that
+    they are file-like objects with no sense of a 'closed' state.
     """
 
+    if isinstance(f, string_types):
+        return True
+
     if hasattr(f, 'closed'):
         return f.closed
     elif hasattr(f, 'fileobj') and hasattr(f.fileobj, 'closed'):
@@ -466,15 +467,25 @@ def fileobj_mode(f):
 
     # Go from most to least specific--for example gzip objects have a 'mode'
     # attribute, but it's not analogous to the file.mode attribute
+
+    # gzip.GzipFile -like
     if hasattr(f, 'fileobj') and hasattr(f.fileobj, 'mode'):
         fileobj = f.fileobj
+
+    # astropy.io.fits._File -like, doesn't need additional checks because it's
+    # already validated
     elif hasattr(f, 'fileobj_mode'):
-        # Specifically for astropy.io.fits.file._File objects
         return f.fileobj_mode
+
+    # PIL-Image -like investigate the fp (filebuffer)
     elif hasattr(f, 'fp') and hasattr(f.fp, 'mode'):
         fileobj = f.fp
+
+    # FILEIO -like (normal open(...)), keep as is.
     elif hasattr(f, 'mode'):
         fileobj = f
+
+    # Doesn't look like a file-like object, for example strings, urls or paths.
     else:
         return None
 
@@ -485,173 +496,27 @@ def _fileobj_normalize_mode(f):
     """Takes care of some corner cases in Python where the mode string
     is either oddly formatted or does not truly represent the file mode.
     """
-
-    # I've noticed that sometimes Python can produce modes like 'r+b' which I
-    # would consider kind of a bug--mode strings should be normalized.  Let's
-    # normalize it for them:
     mode = f.mode
 
-    if isinstance(f, _GZIP_FILE_TYPES):
+    # Special case: Gzip modes:
+    if isinstance(f, gzip.GzipFile):
         # GzipFiles can be either readonly or writeonly
-        if mode == _system_gzip.READ:
+        if mode == gzip.READ:
             return 'rb'
-        elif mode == _system_gzip.WRITE:
+        elif mode == gzip.WRITE:
             return 'wb'
         else:
-            # This shouldn't happen?
-            return None
+            return None  # This shouldn't happen?
 
+    # Sometimes Python can produce modes like 'r+b' which will be normalized
+    # here to 'rb+'
     if '+' in mode:
         mode = mode.replace('+', '')
         mode += '+'
 
-    if _fileobj_is_append_mode(f) and 'a' not in mode:
-        mode = mode.replace('r', 'a').replace('w', 'a')
-
     return mode
 
 
-def _fileobj_is_append_mode(f):
-    """Normally the way to tell if a file is in append mode is if it has
-    'a' in the mode string.  However on Python 3 (or in particular with
-    the io module) this can't be relied on.  See
-    http://bugs.python.org/issue18876.
-    """
-
-    if 'a' in f.mode:
-        # Take care of the obvious case first
-        return True
-
-    # We might have an io.FileIO in which case the only way to know for sure
-    # if the file is in append mode is to ask the file descriptor
-    if not hasattr(f, 'fileno'):
-        # Who knows what this is?
-        return False
-
-    # Call platform-specific _is_append_mode
-    # If this file is already closed this can result in an error
-    try:
-        return _is_append_mode_platform(f.fileno())
-    except (ValueError, IOError):
-        return False
-
-
-if sys.platform.startswith('win32'):
-    # This global variable is used in _is_append_mode to cache the computed
-    # size of the ioinfo struct from msvcrt which may have a different size
-    # depending on the version of the library and how it was compiled
-    _sizeof_ioinfo = None
-
-    def _make_is_append_mode():
-        # We build the platform-specific _is_append_mode function for Windows
-        # inside a function factory in order to avoid cluttering the local
-        # namespace with ctypes stuff
-        from ctypes import (cdll, c_size_t, c_void_p, c_int, c_char,
-                            Structure, POINTER, cast)
-
-        try:
-            from ctypes.util import find_msvcrt
-        except ImportError:
-            # find_msvcrt is not available on Python 2.5 so we have to provide
-            # it ourselves anyways
-            from distutils.msvccompiler import get_build_version
-
-            def find_msvcrt():
-                version = get_build_version()
-                if version is None:
-                    # better be safe than sorry
-                    return None
-                if version <= 6:
-                    clibname = 'msvcrt'
-                else:
-                    clibname = 'msvcr%d' % (version * 10)
-
-                # If python was built with in debug mode
-                import imp
-                if imp.get_suffixes()[0][0] == '_d.pyd':
-                    clibname += 'd'
-                return clibname+'.dll'
-
-        def _dummy_is_append_mode(fd):
-            warnings.warn(
-                'Could not find appropriate MS Visual C Runtime '
-                'library or library is corrupt/misconfigured; cannot '
-                'determine whether your file object was opened in append '
-                'mode.  Please consider using a file object opened in write '
-                'mode instead.')
-            return False
-
-        msvcrt_dll = find_msvcrt()
-        if msvcrt_dll is None:
-            # If for some reason the C runtime can't be located then we're dead
-            # in the water.  Just return a dummy function
-            return _dummy_is_append_mode
-
-        msvcrt = cdll.LoadLibrary(msvcrt_dll)
-
-
-        # Constants
-        IOINFO_L2E = 5
-        IOINFO_ARRAY_ELTS = 1 << IOINFO_L2E
-        IOINFO_ARRAYS = 64
-        FAPPEND = 0x20
-        _NO_CONSOLE_FILENO = -2
-
-
-        # Types
-        intptr_t = POINTER(c_int)
-
-        class my_ioinfo(Structure):
-            _fields_ = [('osfhnd', intptr_t),
-                        ('osfile', c_char)]
-
-        # Functions
-        _msize = msvcrt._msize
-        _msize.argtypes = (c_void_p,)
-        _msize.restype = c_size_t
-
-        # Variables
-        # Since we don't know how large the ioinfo struct is just treat the
-        # __pioinfo array as an array of byte pointers
-        __pioinfo = cast(msvcrt.__pioinfo, POINTER(POINTER(c_char)))
-
-        # Determine size of the ioinfo struct; see the comment above where
-        # _sizeof_ioinfo = None is set
-        global _sizeof_ioinfo
-        if __pioinfo[0] is not None:
-            _sizeof_ioinfo = _msize(__pioinfo[0]) // IOINFO_ARRAY_ELTS
-
-        if not _sizeof_ioinfo:
-            # This shouldn't happen, but I suppose it could if one is using a
-            # broken msvcrt, or just happened to have a dll of the same name
-            # lying around.
-            return _dummy_is_append_mode
-
-        def _is_append_mode(fd):
-            global _sizeof_ioinfo
-            if fd != _NO_CONSOLE_FILENO:
-                idx1 = fd >> IOINFO_L2E # The index into the __pioinfo array
-                # The n-th ioinfo pointer in __pioinfo[idx1]
-                idx2 = fd & ((1 << IOINFO_L2E) - 1)
-                if 0 <= idx1 < IOINFO_ARRAYS and __pioinfo[idx1] is not None:
-                    # Doing pointer arithmetic in ctypes is irritating
-                    pio = c_void_p(cast(__pioinfo[idx1], c_void_p).value +
-                                   idx2 * _sizeof_ioinfo)
-                    ioinfo = cast(pio, POINTER(my_ioinfo)).contents
-                    return bool(ord(ioinfo.osfile) & FAPPEND)
-            return False
-
-        return _is_append_mode
-
-    _is_append_mode_platform = _make_is_append_mode()
-    del _make_is_append_mode
-else:
-    import fcntl
-
-    def _is_append_mode_platform(fd):
-        return bool(fcntl.fcntl(fd, fcntl.F_GETFL) & os.O_APPEND)
-
-
 def fileobj_is_binary(f):
     """
     Returns True if the give file or file-like object has a file open in binary
@@ -673,16 +538,7 @@ def fileobj_is_binary(f):
         return True
 
 
-if six.PY3:
-    maketrans = str.maketrans
-
-    def translate(s, table, deletechars):
-        if deletechars:
-            table = table.copy()
-            for c in deletechars:
-                table[ord(c)] = None
-        return s.translate(table)
-elif six.PY2:
+if six.PY2:
     maketrans = string.maketrans
 
     def translate(s, table, deletechars):
@@ -700,9 +556,18 @@ elif six.PY2:
             for c in deletechars:
                 table[ord(c)] = None
             return s.translate(table)
+else:
+    maketrans = str.maketrans
+
+    def translate(s, table, deletechars):
+        if deletechars:
+            table = table.copy()
+            for c in deletechars:
+                table[ord(c)] = None
+        return s.translate(table)
 
 
-def fill(text, width, *args, **kwargs):
+def fill(text, width, **kwargs):
     """
     Like :func:`textwrap.wrap` but preserves existing paragraphs which
     :func:`textwrap.wrap` does not otherwise handle well.  Also handles section
@@ -715,7 +580,7 @@ def fill(text, width, *args, **kwargs):
         if all(len(l) < width for l in t.splitlines()):
             return t
         else:
-            return textwrap.fill(t, width, *args, **kwargs)
+            return textwrap.fill(t, width, **kwargs)
 
     return '\n\n'.join(maybe_fill(p) for p in paragraphs)
 
@@ -727,7 +592,8 @@ def fill(text, width, *args, **kwargs):
 # time it is needed.
 CHUNKED_FROMFILE = None
 
-def _array_from_file(infile, dtype, count, sep):
+
+def _array_from_file(infile, dtype, count):
     """Create a numpy array from a file or a file-like object."""
 
     if isfile(infile):
@@ -743,27 +609,32 @@ def _array_from_file(infile, dtype, count, sep):
         if CHUNKED_FROMFILE:
             chunk_size = int(1024 ** 3 / dtype.itemsize)  # 1Gb to be safe
             if count < chunk_size:
-                return np.fromfile(infile, dtype=dtype, count=count, sep=sep)
+                return np.fromfile(infile, dtype=dtype, count=count)
             else:
                 array = np.empty(count, dtype=dtype)
                 for beg in range(0, count, chunk_size):
                     end = min(count, beg + chunk_size)
-                    array[beg:end] = np.fromfile(infile, dtype=dtype, count=end - beg, sep=sep)
+                    array[beg:end] = np.fromfile(infile, dtype=dtype, count=end - beg)
                 return array
         else:
-            return np.fromfile(infile, dtype=dtype, count=count, sep=sep)
+            return np.fromfile(infile, dtype=dtype, count=count)
     else:
         # treat as file-like object with "read" method; this includes gzip file
         # objects, because numpy.fromfile just reads the compressed bytes from
         # their underlying file object, instead of the decompressed bytes
         read_size = np.dtype(dtype).itemsize * count
         s = infile.read(read_size)
-        return np.fromstring(s, dtype=dtype, count=count, sep=sep)
+        array = np.frombuffer(s, dtype=dtype, count=count)
+        # copy is needed because np.frombuffer returns a read-only view of the
+        # underlying buffer
+        array = array.copy()
+        return array
 
 
 _OSX_WRITE_LIMIT = (2 ** 32) - 1
 _WIN_WRITE_LIMIT = (2 ** 31) - 1
 
+
 def _array_to_file(arr, outfile):
     """
     Write a numpy array to a file or a file-like object.
@@ -781,8 +652,7 @@ def _array_to_file(arr, outfile):
     `ndarray.tofile`.  Otherwise a slower Python implementation is used.
     """
 
-
-    if isfile(outfile):
+    if isfile(outfile) and not isinstance(outfile, io.BufferedIOBase):
         write = lambda a, f: a.tofile(f)
     else:
         write = _array_to_file_like
@@ -810,7 +680,7 @@ def _array_to_file(arr, outfile):
     # Write one chunk at a time for systems whose fwrite chokes on large
     # writes.
     idx = 0
-    arr = arr.view(type=np.ndarray).flatten()
+    arr = arr.view(np.ndarray).flatten()
     while idx < arr.nbytes:
         write(arr[idx:idx + chunksize], outfile)
         idx += chunksize
@@ -822,12 +692,26 @@ def _array_to_file_like(arr, fileobj):
     `numpy.ndarray.tofile`).
     """
 
+    # If the array is empty, we can simply take a shortcut and return since
+    # there is nothing to write.
+    if len(arr) == 0:
+        return
+
     if arr.flags.contiguous:
+
         # It suffices to just pass the underlying buffer directly to the
-        # fileobj's write (assuming it supports the buffer interface, which
-        # unfortunately there's no simple way to check)
-        fileobj.write(arr.data)
-    elif hasattr(np, 'nditer'):
+        # fileobj's write (assuming it supports the buffer interface). If
+        # it does not have the buffer interface, a TypeError should be returned
+        # in which case we can fall back to the other methods.
+
+        try:
+            fileobj.write(arr.data)
+        except TypeError:
+            pass
+        else:
+            return
+
+    if hasattr(np, 'nditer'):
         # nditer version for non-contiguous arrays
         for item in np.nditer(arr):
             fileobj.write(item.tostring())
@@ -862,6 +746,7 @@ def _write_string(f, s):
     elif isinstance(f, StringIO) and isinstance(s, np.ndarray):
         # Workaround for StringIO/ndarray incompatibility
         s = s.data
+
     f.write(s)
 
 
@@ -899,7 +784,7 @@ def _is_pseudo_unsigned(dtype):
 
 
 def _is_int(val):
-    return isinstance(val, integer_types + (np.integer,))
+    return isinstance(val, all_integer_types)
 
 
 def _str_to_num(val):
@@ -916,15 +801,15 @@ def _str_to_num(val):
 def _words_group(input, strlen):
     """
     Split a long string into parts where each part is no longer
-    than `strlen` and no word is cut into two pieces.  But if
-    there is one single word which is longer than `strlen`, then
+    than ``strlen`` and no word is cut into two pieces.  But if
+    there is one single word which is longer than ``strlen``, then
     it will be split in the middle of the word.
     """
 
     words = []
     nblanks = input.count(' ')
     nmax = max(nblanks, len(input) // strlen + 1)
-    arr = np.fromstring((input + ' '), dtype=(binary_type, 1))
+    arr = np.frombuffer((input + ' ').encode('utf8'), dtype=(binary_type, 1))
 
     # locations of the blanks
     blank_loc = np.nonzero(arr == b' ')[0]
@@ -936,7 +821,7 @@ def _words_group(input, strlen):
             offset = blank_loc[loc - 1] + 1
             if loc == 0:
                 offset = -1
-        except:
+        except Exception:
             offset = len(input)
 
         # check for one word longer than strlen, break in the middle
@@ -979,3 +864,61 @@ def _get_array_mmap(array):
         if isinstance(base.base, mmap.mmap):
             return base.base
         base = base.base
+
+
+@contextmanager
+def _free_space_check(hdulist, dirname=None):
+    try:
+        yield
+    except OSError as exc:
+        error_message = ''
+        if not isinstance(hdulist, list):
+            hdulist = [hdulist, ]
+        if dirname is None:
+            dirname = os.path.dirname(hdulist._file.name)
+        if os.path.isdir(dirname):
+            free_space = data.get_free_space_in_dir(dirname)
+            hdulist_size = sum(hdu.size for hdu in hdulist)
+            if free_space < hdulist_size:
+                error_message = ("Not enough space on disk: requested {}, "
+                                 "available {}. ".format(hdulist_size, free_space))
+
+        for hdu in hdulist:
+            hdu._close()
+
+        raise OSError(error_message + str(exc))
+
+
+def _extract_number(value, default):
+    """
+    Attempts to extract an integer number from the given value. If the
+    extraction fails, the value of the 'default' argument is returned.
+    """
+
+    try:
+        # The _str_to_num method converts the value to string/float
+        # so we need to perform one additional conversion to int on top
+        return int(_str_to_num(value))
+    except (TypeError, ValueError):
+        return default
+
+
+def get_testdata_filepath(filename):
+    """
+    Return a string representing the path to the file requested from the
+    io.fits test data set.
+
+    .. versionadded:: 2.0.3
+
+    Parameters
+    ----------
+    filename : str
+        The filename of the test data file.
+
+    Returns
+    -------
+    filepath : str
+        The path to the requested file.
+    """
+    return data.get_pkg_data_filename(
+        'io/fits/tests/data/{}'.format(filename), 'astropy')
