@@ -8,15 +8,16 @@ from __future__ import (absolute_import, division, print_function,
 
 import itertools
 
+import pytest
 import numpy as np
-from numpy.testing import utils
+from numpy.testing import (assert_allclose, assert_equal, assert_array_equal,
+                           assert_almost_equal)
 
 from . import irafutil
 from .. import models, fitting
-from ..core import Model, FittableModel, ModelDefinitionError
+from ..core import Model, FittableModel
 from ..parameters import Parameter, InputParameterError
 from ...utils.data import get_pkg_data_filename
-from ...tests.helper import pytest
 
 
 def setter1(val):
@@ -37,7 +38,7 @@ class SetterModel(FittableModel):
     yc = Parameter(default=1, setter=setter2)
 
     def __init__(self, xc, yc, p):
-        self.p = p # p is a value intended to be used by the setter
+        self.p = p  # p is a value intended to be used by the setter
         super(SetterModel, self).__init__()
         self.xc = xc
         self.yc = yc
@@ -85,23 +86,23 @@ def test_parameter_properties():
     with pytest.raises(AttributeError):
         p.name = 'beta'
 
-    assert p.fixed == False
+    assert p.fixed is False
     p.fixed = True
-    assert p.fixed == True
+    assert p.fixed is True
 
-    assert p.tied == False
+    assert p.tied is False
     p.tied = lambda _: 0
 
     p.tied = False
-    assert p.tied == False
+    assert p.tied is False
 
-    assert p.min == None
+    assert p.min is None
     p.min = 42
     assert p.min == 42
     p.min = None
-    assert p.min == None
+    assert p.min is None
 
-    assert p.max == None
+    assert p.max is None
     # TODO: shouldn't setting a max < min give an error?
     p.max = 41
     assert p.max == 41
@@ -244,27 +245,27 @@ class TestParameters(object):
 
         new_model = self.linear_fitter(self.model, self.x, self.y)
         print(self.y, self.x)
-        utils.assert_allclose(new_model.parameters,
-                              np.array(
-                                  [4826.1066602783685, 952.8943813407858,
-                                   12.641236013982386,
-                                   -1.7910672553339604,
-                                   0.90252884366711317]),
-                              rtol=10 ** (-2))
+        assert_allclose(new_model.parameters,
+                        np.array(
+                            [4826.1066602783685, 952.8943813407858,
+                             12.641236013982386,
+                             -1.7910672553339604,
+                             0.90252884366711317]),
+                        rtol=10 ** (-2))
 
     def testPolynomial1D(self):
         d = {'c0': 11, 'c1': 12, 'c2': 13, 'c3': 14}
         p1 = models.Polynomial1D(3, **d)
-        utils.assert_equal(p1.parameters, [11, 12, 13, 14])
+        assert_equal(p1.parameters, [11, 12, 13, 14])
 
     def test_poly1d_multiple_sets(self):
         p1 = models.Polynomial1D(3, n_models=3)
-        utils.assert_equal(p1.parameters, [0.0, 0.0, 0.0, 0, 0, 0,
-                                           0, 0, 0, 0, 0, 0])
-        utils.assert_array_equal(p1.c0, [0, 0, 0])
+        assert_equal(p1.parameters, [0.0, 0.0, 0.0, 0, 0, 0,
+                                     0, 0, 0, 0, 0, 0])
+        assert_array_equal(p1.c0, [0, 0, 0])
         p1.c0 = [10, 10, 10]
-        utils.assert_equal(p1.parameters, [10.0, 10.0, 10.0, 0, 0,
-                                           0, 0, 0, 0, 0, 0, 0])
+        assert_equal(p1.parameters, [10.0, 10.0, 10.0, 0, 0,
+                                     0, 0, 0, 0, 0, 0, 0])
 
     def test_par_slicing(self):
         """
@@ -272,20 +273,20 @@ class TestParameters(object):
         """
         p1 = models.Polynomial1D(3, n_models=3)
         p1.c0[:2] = [10, 10]
-        utils.assert_equal(p1.parameters, [10.0, 10.0, 0.0, 0, 0,
-                                           0, 0, 0, 0, 0, 0, 0])
+        assert_equal(p1.parameters, [10.0, 10.0, 0.0, 0, 0,
+                                     0, 0, 0, 0, 0, 0, 0])
 
     def test_poly2d(self):
         p2 = models.Polynomial2D(degree=3)
         p2.c0_0 = 5
-        utils.assert_equal(p2.parameters, [5, 0, 0, 0, 0, 0, 0, 0, 0, 0])
+        assert_equal(p2.parameters, [5, 0, 0, 0, 0, 0, 0, 0, 0, 0])
 
     def test_poly2d_multiple_sets(self):
         kw = {'c0_0': [2, 3], 'c1_0': [1, 2], 'c2_0': [4, 5],
               'c0_1': [1, 1], 'c0_2': [2, 2], 'c1_1': [5, 5]}
         p2 = models.Polynomial2D(2, **kw)
-        utils.assert_equal(p2.parameters, [2, 3, 1, 2, 4, 5,
-                                           1, 1, 2, 2, 5, 5])
+        assert_equal(p2.parameters, [2, 3, 1, 2, 4, 5,
+                                     1, 1, 2, 2, 5, 5])
 
     def test_shift_model_parameters1d(self):
         sh1 = models.Shift(2)
@@ -297,7 +298,7 @@ class TestParameters(object):
         sc1 = models.Scale([2, 2])
         sc1.factor = [3, 3]
         assert np.all(sc1.factor == [3, 3])
-        utils.assert_array_equal(sc1.factor.value, [3, 3])
+        assert_array_equal(sc1.factor.value, [3, 3])
 
     def test_parameters_wrong_shape(self):
         sh1 = models.Shift(2)
@@ -319,7 +320,7 @@ class TestMultipleParameterSets(object):
         Test that a change to one parameter as a set propagates to param_sets.
         """
         self.gmodel.amplitude = [1, 10]
-        utils.assert_almost_equal(
+        assert_almost_equal(
             self.gmodel.param_sets,
             np.array([[1.,
                        10],
@@ -335,7 +336,7 @@ class TestMultipleParameterSets(object):
         param_sets.
         """
         self.gmodel.amplitude[0] = 11
-        utils.assert_almost_equal(
+        assert_almost_equal(
             self.gmodel.param_sets,
             np.array([[11.,
                        10],
@@ -347,8 +348,8 @@ class TestMultipleParameterSets(object):
 
     def test_change_parameters(self):
         self.gmodel.parameters = [13, 10, 9, 5.2, 0.4, 0.7]
-        utils.assert_almost_equal(self.gmodel.amplitude.value, [13., 10.])
-        utils.assert_almost_equal(self.gmodel.mean.value, [9., 5.2])
+        assert_almost_equal(self.gmodel.amplitude.value, [13., 10.])
+        assert_almost_equal(self.gmodel.mean.value, [9., 5.2])
 
 
 class TestParameterInitialization(object):
@@ -371,7 +372,7 @@ class TestParameterInitialization(object):
         t = TParModel(10, [1, 2])
         assert len(t) == 1
         assert t.model_set_axis is False
-        assert np.issubdtype(t.param_sets.dtype, object)
+        assert np.issubdtype(t.param_sets.dtype, np.object_)
         assert len(t.param_sets) == 2
         assert np.all(t.param_sets[0] == [10])
         assert np.all(t.param_sets[1] == [[1, 2]])
@@ -439,7 +440,7 @@ class TestParameterInitialization(object):
         assert len(t) == 1
         assert t.model_set_axis is False
         assert len(t.param_sets) == 2
-        assert np.issubdtype(t.param_sets.dtype, object)
+        assert np.issubdtype(t.param_sets.dtype, np.object_)
         assert np.all(t.param_sets[0] == [[[10, 20, 30], [40, 50, 60]]])
         assert np.all(t.param_sets[1] == [[1, 2, 3]])
         assert np.all(t.parameters == [10, 20, 30, 40, 50, 60, 1, 2, 3])
@@ -471,10 +472,10 @@ class TestParameterInitialization(object):
         assert len(t) == 2
         assert t.model_set_axis == 0
         assert len(t.param_sets) == 2
-        assert np.issubdtype(t.param_sets.dtype, object)
+        assert np.issubdtype(t.param_sets.dtype, np.object_)
         assert np.all(t.param_sets[0] == [[10], [20]])
         assert np.all(t.param_sets[1] == [[1, 2], [3, 4]])
-        assert np.all(t.parameters  == [10, 20, 1, 2, 3, 4])
+        assert np.all(t.parameters == [10, 20, 1, 2, 3, 4])
         assert t.coeff.shape == ()
         assert t.e.shape == (2,)
 
@@ -489,7 +490,7 @@ class TestParameterInitialization(object):
         assert t.e.shape == (2,)
 
         t2 = TParModel([[10, 20, 30], [40, 50, 60]],
-                          [[1, 2, 3], [4, 5, 6]], n_models=2)
+                       [[1, 2, 3], [4, 5, 6]], n_models=2)
         assert len(t2) == 2
         assert t2.model_set_axis == 0
         assert np.all(t2.param_sets == [[[10, 20, 30], [40, 50, 60]],
@@ -503,14 +504,14 @@ class TestParameterInitialization(object):
         with pytest.raises(InputParameterError):
             # Can't broadcast different array shapes
             TParModel([[[1, 2], [3, 4]], [[5, 6], [7, 8]]],
-                         [[9, 10, 11], [12, 13, 14]], n_models=2)
+                      [[9, 10, 11], [12, 13, 14]], n_models=2)
 
         t = TParModel([[[10, 20], [30, 40]], [[50, 60], [70, 80]]],
-                         [[1, 2], [3, 4]], n_models=2)
+                      [[1, 2], [3, 4]], n_models=2)
         assert len(t) == 2
         assert t.model_set_axis == 0
         assert len(t.param_sets) == 2
-        assert np.issubdtype(t.param_sets.dtype, object)
+        assert np.issubdtype(t.param_sets.dtype, np.object_)
         assert np.all(t.param_sets[0] == [[[10, 20], [30, 40]],
                                           [[50, 60], [70, 80]]])
         assert np.all(t.param_sets[1] == [[[1, 2]], [[3, 4]]])
@@ -521,7 +522,7 @@ class TestParameterInitialization(object):
 
     def test_two_model_2d_array_parameters(self):
         t = TParModel([[[10, 20], [30, 40]], [[50, 60], [70, 80]]],
-                         [[[1, 2], [3, 4]], [[5, 6], [7, 8]]], n_models=2)
+                      [[[1, 2], [3, 4]], [[5, 6], [7, 8]]], n_models=2)
         assert len(t) == 2
         assert t.model_set_axis == 0
         assert np.all(t.param_sets == [[[[10, 20], [30, 40]],
@@ -544,7 +545,7 @@ class TestParameterInitialization(object):
         assert len(t) == 2
         assert t.model_set_axis == -1
         assert len(t.param_sets) == 2
-        assert np.issubdtype(t.param_sets.dtype, object)
+        assert np.issubdtype(t.param_sets.dtype, np.object_)
         assert np.all(t.param_sets[0] == [[[10, 50], [20, 60]],
                                           [[30, 70], [40, 80]]])
         assert np.all(t.param_sets[1] == [[[1, 3], [2, 4]]])
@@ -572,7 +573,7 @@ class TestParameterInitialization(object):
     def test_array_parameter2(self):
         with pytest.raises(InputParameterError):
             m = TParModel(np.array([[1, 2], [3, 4]]), (1, 1, 11),
-                             model_set_axis=0)
+                          model_set_axis=0)
 
     def test_array_parameter4(self):
         """
@@ -584,7 +585,7 @@ class TestParameterInitialization(object):
         assert len(t4) == 1
         assert t4.coeff.shape == (2, 2)
         assert t4.e.shape == (2,)
-        assert np.issubdtype(t4.param_sets.dtype, object)
+        assert np.issubdtype(t4.param_sets.dtype, np.object_)
         assert np.all(t4.param_sets[0] == [[1, 2], [3, 4]])
         assert np.all(t4.param_sets[1] == [5, 6])
 
@@ -615,11 +616,11 @@ def test_non_broadcasting_parameters():
 
 
 def test_setter():
-    pars = np.random.rand(20).reshape((10,2))
+    pars = np.random.rand(20).reshape((10, 2))
 
     model = SetterModel(-1, 3, np.pi)
 
     for x, y in pars:
         model.x = x
         model.y = y
-        utils.assert_almost_equal(model(x, y), (x + 1)**2 + (y - np.pi * 3)**2)
+        assert_almost_equal(model(x, y), (x + 1)**2 + (y - np.pi * 3)**2)
