@@ -3,7 +3,7 @@ from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
 import os
-
+import warnings
 import pytest
 import numpy as np
 
@@ -11,14 +11,15 @@ from ....tests.helper import assert_quantity_allclose, catch_warnings, remote_da
 from .. import iers
 from .... import units as u
 from ....table import QTable
-from ....time import Time
+from ....time import Time, TimeDelta
 from ....extern.six.moves import urllib
+from ....utils.exceptions import AstropyWarning
 
 
 FILE_NOT_FOUND_ERROR = getattr(__builtins__, 'FileNotFoundError', IOError)
 
 try:
-    iers.IERS_A.open()  # check if IERS_A is available
+    iers.IERS_A.open('finals2000A.all')  # check if IERS_A is available
 except IOError:
     HAS_IERS_A = False
 else:
@@ -134,6 +135,9 @@ class TestIERS_AExcerpt():
 class TestIERS_A():
 
     def test_simple(self):
+        """Test that open() by default reads a 'finals2000A.all' file."""
+        # Ensure we remove any cached table (gh-5131).
+        iers.IERS_A.close()
         iers_tab = iers.IERS_A.open()
         jd1 = np.array([2456108.5, 2456108.5, 2456108.5, 2456109.5, 2456109.5])
         jd2 = np.array([0.49999421, 0.99997685, 0.99998843, 0., 0.5])
@@ -154,6 +158,60 @@ class TestIERS_A():
 
 class TestIERS_Auto():
 
+    def setup_class(self):
+        """Set up useful data for the tests.
+        """
+        self.N = 40
+        self.ame = 30.0
+        self.iers_a_file_1 = os.path.join(os.path.dirname(__file__), 'finals2000A-2016-02-30-test')
+        self.iers_a_file_2 = os.path.join(os.path.dirname(__file__), 'finals2000A-2016-04-30-test')
+        self.iers_a_url_1 = os.path.normpath('file://' + os.path.abspath(self.iers_a_file_1))
+        self.iers_a_url_2 = os.path.normpath('file://' + os.path.abspath(self.iers_a_file_2))
+        self.t = Time.now() + TimeDelta(10, format='jd') * np.arange(self.N)
+
+    def teardown_method(self, method):
+        """Run this after every test.
+        """
+        iers.IERS_Auto.close()
+
+    def test_interpolate_error_formatting(self):
+        """Regression test: make sure the error message in
+        IERS_Auto._check_interpolate_indices() is formatted correctly.
+        """
+        with iers.conf.set_temp('iers_auto_url', self.iers_a_url_1):
+            with iers.conf.set_temp('auto_max_age', self.ame):
+                with pytest.raises(ValueError) as err:
+                    iers_table = iers.IERS_Auto.open()
+                    with warnings.catch_warnings():
+                        # Ignoring this if it comes up -- IERS_Auto predictive
+                        # values are older than 30.0 days but downloading the
+                        # latest table did not find newer values
+                        warnings.simplefilter('ignore', iers.IERSStaleWarning)
+                        iers_table.ut1_utc(self.t.jd1, self.t.jd2)
+        assert str(err.value) == iers.INTERPOLATE_ERROR.format(self.ame)
+
+    def test_auto_max_age_none(self):
+        """Make sure that iers.INTERPOLATE_ERROR's advice about setting
+        auto_max_age = None actually works.
+        """
+        with iers.conf.set_temp('iers_auto_url', self.iers_a_url_1):
+            with iers.conf.set_temp('auto_max_age', None):
+                iers_table = iers.IERS_Auto.open()
+                delta = iers_table.ut1_utc(self.t.jd1, self.t.jd2)
+        assert isinstance(delta, np.ndarray)
+        assert delta.shape == (self.N,)
+        assert_quantity_allclose(delta, np.array([-0.2246227]*self.N)*u.s)
+
+    def test_auto_max_age_minimum(self):
+        """Check that the minimum auto_max_age is enforced.
+        """
+        with iers.conf.set_temp('iers_auto_url', self.iers_a_url_1):
+            with iers.conf.set_temp('auto_max_age', 5.0):
+                with pytest.raises(ValueError) as err:
+                    iers_table = iers.IERS_Auto.open()
+                    delta = iers_table.ut1_utc(self.t.jd1, self.t.jd2)
+        assert str(err.value) == 'IERS auto_max_age configuration value must be larger than 10 days'
+
     @remote_data
     def test_no_auto_download(self):
         with iers.conf.set_temp('auto_download', False):
@@ -162,12 +220,8 @@ class TestIERS_Auto():
 
     @remote_data
     def test_simple(self):
-        iers_a_file_1 = os.path.join(os.path.dirname(__file__), 'finals2000A-2016-02-30-test')
-        iers_a_file_2 = os.path.join(os.path.dirname(__file__), 'finals2000A-2016-04-30-test')
-        iers_a_url_1 = os.path.normpath('file://' + os.path.abspath(iers_a_file_1))
-        iers_a_url_2 = os.path.normpath('file://' + os.path.abspath(iers_a_file_2))
 
-        with iers.conf.set_temp('iers_auto_url', iers_a_url_1):
+        with iers.conf.set_temp('iers_auto_url', self.iers_a_url_1):
 
             dat = iers.IERS_Auto.open()
             assert dat['MJD'][0] == 57359.0 * u.d
@@ -212,7 +266,7 @@ class TestIERS_Auto():
         # 60 days and see that things work.  dat._time_now is still the same value
         # as before, i.e. right around the start of predictive values for the new file.
         # (In other words this is like downloading the latest file online right now).
-        with iers.conf.set_temp('iers_auto_url', iers_a_url_2):
+        with iers.conf.set_temp('iers_auto_url', self.iers_a_url_2):
 
             # Look at times before and after the test file begins.  This forces a new download.
             assert np.allclose(dat.ut1_utc(Time(50000, format='mjd').jd).value, 0.1292905)
